@@ -23,15 +23,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include <glib/gstdio.h>
-#include <glib/gprintf.h>
-#include <glib/gi18n.h>
-
 #include "datasource_gps.h"
 #include "viking.h"
 #include "babel.h"
@@ -145,8 +140,8 @@ gchar* datasource_gps_get_protocol ( gpointer user_data )
   // Uses the list of supported devices
   gps_user_data_t *w = (gps_user_data_t *)user_data;
   last_active = gtk_combo_box_get_active(GTK_COMBO_BOX(w->proto_b));
-  if (a_babel_device_list) {
-    gchar *protocol = ((BabelDevice*)g_list_nth_data(a_babel_device_list, last_active))->name;
+  if ( a_babel_device_list_get() ) {
+    gchar *protocol = ((BabelDevice*)g_list_nth_data(a_babel_device_list_get(), last_active))->name;
     a_settings_set_string ( VIK_SETTINGS_GPS_PROTOCOL, protocol );
     return protocol;
   }
@@ -226,7 +221,7 @@ static void datasource_gps_get_process_options ( gpointer user_data, ProcessOpti
   if (gps_acquire_in_progress) {
     po->babelargs = po->filename = NULL;
   }
-  
+
   gps_acquire_in_progress = TRUE;
 
   device = datasource_gps_get_protocol ( user_data );
@@ -255,7 +250,7 @@ static void datasource_gps_get_process_options ( gpointer user_data, ProcessOpti
 
   po->filename = g_strdup(datasource_gps_get_descriptor(user_data));
 
-  g_debug(_("using cmd '%s' and file '%s'\n"), po->babelargs, po->filename);
+  g_debug(_("using cmd '%s' and file '%s'"), po->babelargs, po->filename);
 }
 
 /**
@@ -285,11 +280,11 @@ static void datasource_gps_off ( gpointer user_data, gchar **babelargs, gchar **
   if (!datasource_gps_get_off ( user_data )){
     return;
   }
-  
-  if (!a_babel_device_list)
+
+  if (!a_babel_device_list_get())
     return;
   last_active = gtk_combo_box_get_active(GTK_COMBO_BOX(w->proto_b));
-  device = ((BabelDevice*)g_list_nth_data(a_babel_device_list, last_active))->name;
+  device = ((BabelDevice*)g_list_nth_data(a_babel_device_list_get(), last_active))->name;
   if (!strcmp(device, "garmin")) {
     device = "garmin,power_off";
   }
@@ -303,7 +298,7 @@ static void datasource_gps_off ( gpointer user_data, gchar **babelargs, gchar **
   *babelargs = g_strdup_printf("-i %s", device);
   /* device points to static content => no free */
   device = NULL;
-  
+
 #if GTK_CHECK_VERSION (2, 24, 0)
   ser = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(w->ser_b));
 #else
@@ -431,8 +426,8 @@ static gboolean show_gps_status(acq_dialog_widgets_t *w)
   return FALSE;
 }
 
-/* 
- * This routine relies on gpsbabel's diagnostic output to display the progress information. 
+/**
+ * This routine relies on gpsbabel's diagnostic output to display the progress information.
  * These outputs differ when different GPS devices are used, so we will need to test
  * them on several and add the corresponding support.
  */
@@ -518,7 +513,7 @@ static void datasource_gps_progress ( BabelProgressCode c, gpointer data, acq_di
       int lsb, msb, cnt;
 
       if (strlen(line) > 20) {
-       sscanf(line+17, "%x", &lsb); 
+       sscanf(line+17, "%x", &lsb);
        sscanf(line+20, "%x", &msb);
        cnt = lsb + msb * 256;
        if ( gps_data->progress_type == RTE ) {
@@ -583,7 +578,7 @@ static void datasource_gps_add_setup_widgets ( GtkWidget *dialog, VikViewport *v
 
   w->proto_l = gtk_label_new (_("GPS Protocol:"));
   w->proto_b = vik_combo_box_text_new ();
-  g_list_foreach (a_babel_device_list, append_element, w->proto_b);
+  g_list_foreach ( a_babel_device_list_get(), append_element, w->proto_b );
 
   if ( last_active < 0 ) {
     find_entry = -1;
@@ -592,11 +587,11 @@ static void datasource_gps_add_setup_widgets ( GtkWidget *dialog, VikViewport *v
     if ( a_settings_get_string ( VIK_SETTINGS_GPS_PROTOCOL, &protocol ) ) {
       // Use setting
       if ( protocol )
-        g_list_foreach (a_babel_device_list, find_protocol, protocol);
+        g_list_foreach ( a_babel_device_list_get(), find_protocol, protocol );
     }
     else {
       // Attempt to maintain default to Garmin devices (assumed most popular/numerous device)
-      g_list_foreach (a_babel_device_list, find_protocol, "garmin");
+      g_list_foreach ( a_babel_device_list_get(), find_protocol, "garmin" );
     }
     g_free ( protocol );
     // If not found set it to the first entry, otherwise use the entry
