@@ -19,14 +19,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <glib/gprintf.h>
-#include <glib/gi18n.h>
-#include <glib/gstdio.h>
-
 #include <expat.h>
 
 #include "viking.h"
@@ -389,11 +381,8 @@ static GList *select_from_list (GtkWindow *parent, GList *list, const gchar *tit
 													 NULL);
 	/* When something is selected then OK */
 	gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
-	GtkWidget *response_w = NULL;
-#if GTK_CHECK_VERSION (2, 20, 0)
 	/* Default to not apply - as initially nothing is selected! */
-	response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
-#endif
+	GtkWidget *response_w = gtk_dialog_get_widget_for_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
 	GtkWidget *label = gtk_label_new ( msg );
 	GtkTreeStore *store = gtk_tree_store_new ( 6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN );
 	GList *list_runner = list;
@@ -462,7 +451,8 @@ static GList *select_from_list (GtkWindow *parent, GList *list, const gchar *tit
 	gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), scrolledwindow, TRUE, TRUE, 0);
 
 	// Ensure a reasonable number of items are shown, but let the width be automatically sized
-	gtk_widget_set_size_request ( dialog, -1, 400) ;
+	gdouble scale = vik_viewport_get_scale ( NULL );
+	gtk_widget_set_size_request ( dialog, -1, 400*scale );
 	gtk_widget_show_all ( dialog );
 
 	if ( response_w )
@@ -573,7 +563,7 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOpti
 #endif
 
 	// Support .zip + bzip2 files directly
-	DownloadFileOptions options = { FALSE, FALSE, NULL, 2, NULL, user_pass, a_try_decompress_file }; // Allow a couple of redirects
+	DownloadFileOptions options = { FALSE, FALSE, NULL, -1, NULL, NULL, 0, NULL, user_pass, FALSE, FALSE, a_try_decompress_file }; // Allow a couple of redirects
 
 	gchar *tmpname = a_download_uri_to_tmp_file ( uri, &options );
 	g_free ( uri );
@@ -699,7 +689,7 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOpti
 	// Would prefer to keep the update in acquire.c,
 	//  however since we may create the layer - need to do the update here
 	if ( got_something )
-		vik_layer_emit_update ( VIK_LAYER(vtl_last) );
+		vik_layer_emit_update ( VIK_LAYER(vtl_last), TRUE );
 
 	// ATM The user is only informed if all getting *all* of the traces failed
 	if ( selected )
