@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
  * Copyright (C) 2012, Guilhem Bonnefille <guilhem.bonnefille@gmail.com>
- * Copyright (C) 2012-2013, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2012-2023, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,36 +20,30 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "viking.h"
 
+#include "fit.h"
 #include "jpg.h"
 #include "gpx.h"
+#include "kml.h"
+#include "tcx.h"
 #include "geojson.h"
 #include "babel.h"
 #include "gpsmapper.h"
+#include "compression.h"
+#include "file_magic.h"
+#include "vikgpslayer.h"
+#include "vikgeocluelayer.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
 
 #include "file.h"
 #include "misc/strtod.h"
 
 #define TEST_BOOLEAN(str) (! ((str)[0] == '\0' || (str)[0] == '0' || (str)[0] == 'n' || (str)[0] == 'N' || (str)[0] == 'f' || (str)[0] == 'F') )
 #define VIK_MAGIC "#VIK"
-#define GPX_MAGIC "<?xm"
-#define VIK_MAGIC_LEN 4
-#define GPX_MAGIC_LEN 4
 
 #define VIKING_FILE_VERSION 1
 
@@ -73,20 +67,19 @@ static void push(Stack **stack)
   *stack = tmp;
 }
 
-static gboolean check_magic ( FILE *f, const gchar *magic_number, guint magic_length )
+gboolean file_check_magic ( FILE *f, const gchar *magic_string )
 {
-  gchar magic[magic_length];
+  gchar magic[strlen(magic_string)];
   gboolean rv = FALSE;
   gint8 i;
   if ( fread(magic, 1, sizeof(magic), f) == sizeof(magic) &&
-      strncmp(magic, magic_number, sizeof(magic)) == 0 )
+      strncmp(magic, magic_string, sizeof(magic)) == 0 )
     rv = TRUE;
   for ( i = sizeof(magic)-1; i >= 0; i-- ) /* the ol' pushback */
     ungetc(magic[i],f);
   return rv;
 }
 
-
 static gboolean str_starts_with ( const gchar *haystack, const gchar *needle, guint16 len_needle, gboolean must_be_longer )
 {
   if ( strlen(haystack) > len_needle - (!must_be_longer) && strncasecmp ( haystack, needle, len_needle ) == 0 )
@@ -94,7 +87,12 @@ static gboolean str_starts_with ( const gchar *haystack, const gchar *needle, gu
   return FALSE;
 }
 
-void file_write_layer_param ( FILE *f, const gchar *name, VikLayerParamType type, VikLayerParamData data ) {
+void file_write_layer_param ( FILE *f, const gchar *name, VikLayerParamType type, VikLayerParamData data )
+{
+  // Skip UI only 'parameters'
+  if ( type == VIK_LAYER_PARAM_SPACER )
+    return;
+
       /* string lists are handled differently. We get a GList (that shouldn't
        * be freed) back for get_param and if it is null we shouldn't write
        * anything at all (otherwise we'd read in a list with an empty string,
@@ -109,7 +107,7 @@ void file_write_layer_param ( FILE *f, const gchar *name, VikLayerParamType type
             iter = iter->next;
           }
         }
-      } else {
+      } else if ( type != VIK_LAYER_PARAM_PTR && type != VIK_LAYER_PARAM_PTR_DEFAULT ) {
         fprintf ( f, "%s=", name );
         switch ( type )
         {
@@ -195,8 +193,8 @@ static void file_write ( VikAggregateLayer *top, FILE *f, gpointer vp, const gch
       vik_viewport_get_draw_centermark(VIK_VIEWPORT(vp)) ? "t" : "f",
       vik_viewport_get_draw_highlight(VIK_VIEWPORT(vp)) ? "t" : "f" );
 
-  if ( ! VIK_LAYER(top)->visible )
-    fprintf ( f, "visible=f\n" );
+  fprintf ( f, "\n~TopLayer %s\n", vik_layer_get_interface(VIK_LAYER(top)->type)->fixed_layer_name );
+  write_layer_params_and_data ( VIK_LAYER(top), f, dirpath );
 
   while (stack && stack->data)
   {
@@ -228,6 +226,8 @@ static void file_write ( VikAggregateLayer *top, FILE *f, gpointer vp, const gch
       }
     }
   }
+
+  fprintf ( f, "~EndTopLayer\n\n" );
 /*
   get vikaggregatelayer's children (?)
   foreach write ALL params,
@@ -261,7 +261,7 @@ static void string_list_set_param (gint i, GList *list, gpointer *layer_and_vp)
   vlsp.is_file_operation   = TRUE;
   vlsp.dirpath             = (const gchar*)layer_and_vp[2];
 
-  vik_layer_set_param ( VIK_LAYER(layer_and_vp[0]), &vlsp );
+  (void)vik_layer_set_param ( VIK_LAYER(layer_and_vp[0]), &vlsp );
 }
 
 /**
@@ -287,6 +287,8 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
   GHashTable *string_lists = g_hash_table_new(g_direct_hash,g_direct_equal);
 
   gboolean successful_read = TRUE;
+  gint toplayer_number = 0;
+  gboolean auto_load_external = FALSE;
 
   push(&stack);
   stack->under = NULL;
@@ -302,7 +304,6 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
 
     if ( line[0] == '#' )
       continue;
-    
 
     len = strlen(line);
     if ( len > 0 && line[len-1] == '\n' )
@@ -319,6 +320,17 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
       line++; len--;
       if ( *line == '\0' )
         continue;
+      else if ( str_starts_with ( line, "TopLayer ", 9, TRUE ) )
+      {
+        toplayer_number++;
+        // No need to create the Top Layer, values replace the current
+        push(&stack);
+        stack->data = (gpointer)top;
+        params = vik_layer_get_interface(VIK_LAYER(top)->type)->params;
+        params_count = vik_layer_get_interface(VIK_LAYER(top)->type)->params_count;
+        auto_load_external = vik_aggregate_layer_get_auto_load_external ( top );
+        continue;
+      }
       else if ( str_starts_with ( line, "Layer ", 6, TRUE ) )
       {
         if ( ! stack->data )
@@ -348,6 +360,14 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
             params = vik_layer_get_interface(type)->params;
             params_count = vik_layer_get_interface(type)->params_count;
           }
+#ifdef HAVE_LIBGEOCLUE_2
+          else if (parent_type == VIK_LAYER_GEOCLUE)
+          {
+            stack->data = (gpointer) vik_geoclue_layer_get_trw(VIK_GEOCLUE_LAYER(stack->under->data));
+            params = vik_layer_get_interface(type)->params;
+            params_count = vik_layer_get_interface(type)->params_count;
+          }
+#endif
           else
           {
             stack->data = (gpointer) vik_layer_create ( type, vp, FALSE );
@@ -377,6 +397,7 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
             if (VIK_LAYER(stack->under->data)->type == VIK_LAYER_AGGREGATE) {
               vik_aggregate_layer_add_layer ( VIK_AGGREGATE_LAYER(stack->under->data), VIK_LAYER(stack->data), FALSE );
               vik_layer_post_read ( VIK_LAYER(stack->data), vp, TRUE );
+              auto_load_external = vik_aggregate_layer_get_auto_load_external ( VIK_AGGREGATE_LAYER(stack->under->data) );
             }
             else if (VIK_LAYER(stack->under->data)->type == VIK_LAYER_GPS) {
               /* TODO: anything else needs to be done here ? */
@@ -394,7 +415,7 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
         if ( stack->data && vik_layer_get_interface(VIK_LAYER(stack->data)->type)->read_file_data )
         {
           /* must read until hits ~EndLayerData */
-          if ( ! vik_layer_get_interface(VIK_LAYER(stack->data)->type)->read_file_data ( VIK_LAYER(stack->data), f, dirpath ) )
+          if ( ! vik_layer_get_interface(VIK_LAYER(stack->data)->type)->read_file_data ( VIK_LAYER(stack->data), f, dirpath, auto_load_external ) )
             successful_read = FALSE;
         }
         else
@@ -416,6 +437,12 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
           continue;
         }
       }
+      else if ( str_starts_with ( line, "EndTopLayer", 11, FALSE ) )
+      {
+        toplayer_number--;
+        if ( stack )
+          pop(&stack);
+      }
       else
       {
         successful_read = FALSE;
@@ -500,7 +527,7 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
             VikLayerParamData x;
             line += eq_pos+1;
             if ( params[i].type == VIK_LAYER_PARAM_STRING_LIST ) {
-              GList *l = g_list_append ( g_hash_table_lookup ( string_lists, GINT_TO_POINTER ((gint) i) ), 
+              GList *l = g_list_append ( g_hash_table_lookup ( string_lists, GINT_TO_POINTER ((gint) i) ),
 					 g_strdup(line) );
               g_hash_table_replace ( string_lists, GINT_TO_POINTER ((gint)i), l );
               /* add the value to a list, possibly making a new list.
@@ -524,7 +551,7 @@ static gboolean file_read ( VikAggregateLayer *top, FILE *f, const gchar *dirpat
               vlsp.vp                  = vp;
               vlsp.is_file_operation   = TRUE;
               vlsp.dirpath             = dirpath;
-              vik_layer_set_param ( VIK_LAYER(stack->data), &vlsp );
+              (void)vik_layer_set_param ( VIK_LAYER(stack->data), &vlsp );
             }
             found_match = TRUE;
             break;
@@ -552,6 +579,17 @@ name=this
 */
   }
 
+  // Missing "~EndTopLayer" at end of file? - e.g. you've been manually messing around with .vik files
+  // Really don't think should have more than one toplayer in a file anyway
+  //  check at least open and close markers evenly matched
+  if ( toplayer_number != 0 ) {
+    g_warning ( "%s: Mismatched number of toplayer markers %d", __FUNCTION__, toplayer_number );
+    if ( toplayer_number > 0 )
+      // Force a pop() otherwise stack->data pointing to wrong place
+      if ( stack )
+        pop(&stack);
+  }
+
   while ( stack )
   {
     if ( stack->under && stack->under->data && stack->data )
@@ -566,7 +604,9 @@ name=this
     vik_viewport_set_center_latlon ( VIK_VIEWPORT(vp), &ll, TRUE );
 
   if ( ( ! VIK_LAYER(top)->visible ) && VIK_LAYER(top)->realized )
-    vik_treeview_item_set_visible ( VIK_LAYER(top)->vt, &(VIK_LAYER(top)->iter), FALSE ); 
+    vik_treeview_item_set_visible ( VIK_LAYER(top)->vt, &(VIK_LAYER(top)->iter), FALSE );
+  if ( VIK_LAYER(top)->realized )
+    vik_treeview_item_set_name ( VIK_LAYER(top)->vt, &(VIK_LAYER(top)->iter), VIK_LAYER(top)->name );
 
   /* delete anything we've forgotten about -- should only happen when file ends before an EndLayer */
   g_hash_table_foreach ( string_lists, string_list_delete, NULL );
@@ -596,6 +636,8 @@ static FILE *xfopen ( const char *fn )
   if ( strcmp(fn,"-") == 0 )
     return stdin;
   else
+    // NB File names passed into Viking should already be converted into UFT8
+    //  by g_win32_get_command_line() in main.c for Windows systems
     return g_fopen(fn, "r");
 }
 
@@ -615,7 +657,7 @@ gboolean check_file_magic_vik ( const gchar *filename )
   gboolean result = FALSE;
   FILE *ff = xfopen ( filename );
   if ( ff ) {
-    result = check_magic ( ff, VIK_MAGIC, VIK_MAGIC_LEN );
+    result = file_check_magic ( ff, VIK_MAGIC );
     xfclose ( ff );
   }
   return result;
@@ -663,46 +705,60 @@ gchar *append_file_ext ( const gchar *filename, VikFileType_t type )
 }
 
 /**
- * a_file_load:
+ * a_file_load_stream:
  *
  */
-VikLoadType_t a_file_load ( VikAggregateLayer *top, VikViewport *vp, VikTrwLayer *vtl, const gchar *filename_or_uri, gboolean new_layer )
+VikLoadType_t a_file_load_stream ( FILE *f,
+                                   const gchar *filename,
+                                   VikAggregateLayer *top,
+                                   VikViewport *vp,
+                                   VikTrwLayer *vtl,
+                                   gboolean new_layer,
+                                   gboolean external,
+                                   const gchar *dirpath,
+                                   const gchar *name )
 {
-  g_return_val_if_fail ( vp != NULL, LOAD_TYPE_READ_FAILURE );
-
-  char *filename = (char *)filename_or_uri;
-  if (strncmp(filename, "file://", 7) == 0) {
-    // Consider replacing this with:
-    // filename = g_filename_from_uri ( entry, NULL, NULL );
-    // Since this doesn't support URIs properly (i.e. will failure if is it has %20 characters in it)
-    filename = filename + 7;
-    g_debug ( "Loading file %s from URI %s", filename, filename_or_uri );
-  }
-  FILE *f = xfopen ( filename );
-
-  if ( ! f )
-    return LOAD_TYPE_READ_FAILURE;
-
   VikLoadType_t load_answer = LOAD_TYPE_OTHER_SUCCESS;
 
-  gchar *absolute = file_realpath_dup ( filename );
-  gchar *dirpath = NULL;
-  if ( absolute )
-    dirpath = g_path_get_dirname ( absolute );
-  g_free ( absolute );
-
   // Attempt loading the primary file type first - our internal .vik file:
-  if ( check_magic ( f, VIK_MAGIC, VIK_MAGIC_LEN ) )
+  if ( file_check_magic ( f, VIK_MAGIC ) )
   {
     if ( file_read ( top, f, dirpath, vp ) )
       load_answer = LOAD_TYPE_VIK_SUCCESS;
     else
       load_answer = LOAD_TYPE_VIK_FAILURE_NON_FATAL;
   }
+  else if ( file_magic_check ( filename, "application/zip", ".zip" ) ) {
+    (void)fclose ( f );
+    load_answer = uncompress_load_zip_file ( filename, top, vp, vtl, new_layer, external, dirpath );
+  }
+  else if ( file_magic_check ( filename, "application/x-bzip2", ".bz2" ) ) {
+    load_answer = uncompress_load_bzip_file ( filename, top, vp, vtl, new_layer, external );
+  }
+  else if ( file_magic_check ( filename, "application/x-xz", ".xz" ) ) {
+    load_answer = uncompress_load_xz_file ( filename, top, vp, vtl, new_layer, external );
+  }
+  else if ( file_magic_check ( filename, "application/x-lzma", ".lzma" ) ) {
+    load_answer = uncompress_load_xz_file ( filename, top, vp, vtl, new_layer, external );
+  }
+  else if ( file_magic_check ( filename, "application/gzip", ".gz" ) ) {
+    load_answer = uncompress_load_gz_file ( filename, top, vp, vtl, new_layer, external );
+  }
   else if ( a_jpg_magic_check ( filename ) ) {
     if ( ! a_jpg_load_file ( top, filename, vp ) )
       load_answer = LOAD_TYPE_UNSUPPORTED_FAILURE;
   }
+  // NB TCX files are XML
+  else if ( a_file_check_ext ( filename, ".tcx" ) && file_check_magic ( f, FILE_XML_MAGIC ) && !external ) {
+    if ( !a_tcx_read_file ( top, vp, f, filename ) ) {
+      load_answer = LOAD_TYPE_TCX_FAILURE;
+    }
+  }
+  else if ( a_fit_check_magic ( f ) && !external ) {
+    if ( !a_fit_read_file ( top, vp, f, filename ) ) {
+      load_answer = LOAD_TYPE_FIT_FAILURE;
+    }
+  }
   else
   {
 	// For all other file types which consist of tracks, routes and/or waypoints,
@@ -728,23 +784,50 @@ VikLoadType_t a_file_load ( VikAggregateLayer *top, VikViewport *vp, VikTrwLayer
 
     if (add_new) {
       vtl = VIK_TRW_LAYER (vik_layer_create ( VIK_LAYER_TRW, vp, FALSE ));
-      vik_layer_rename ( VIK_LAYER(vtl), a_file_basename ( filename ) );
+      vik_layer_rename ( VIK_LAYER(vtl), name ? name : a_file_basename ( filename ) );
     }
 
+    if ( !external )
+      vik_trw_layer_set_filename ( vtl, filename );
+
     // In fact both kml & gpx files start the same as they are in xml
-    if ( a_file_check_ext ( filename, ".kml" ) && check_magic ( f, GPX_MAGIC, GPX_MAGIC_LEN ) ) {
-      // Implicit Conversion
-      ProcessOptions po = { "-i kml", filename, NULL, NULL, NULL, NULL };
-      if ( ! ( success = a_babel_convert_from ( vtl, &po, NULL, NULL, NULL ) ) ) {
-        load_answer = LOAD_TYPE_GPSBABEL_FAILURE;
+    if ( a_file_check_ext ( filename, ".kml" ) && file_check_magic ( f, FILE_XML_MAGIC ) && !external ) {
+      if ( ! ( success = a_kml_read_file ( vtl, f, external ) ) ) {
+        load_answer = LOAD_TYPE_KML_FAILURE;
       }
     }
+    else if ( external && a_file_check_ext(filename, ".tcx") && file_check_magic(f, FILE_XML_MAGIC) ) {
+      if ( (success = a_tcx_read_file_into_layer ( vtl, f, filename )) )
+        trw_layer_replace_external ( vtl, filename );
+      else
+        load_answer = LOAD_TYPE_TCX_FAILURE;
+    }
+    else if ( external && a_file_check_ext(filename, ".kml") && file_check_magic(f, FILE_XML_MAGIC) ) {
+      if ( (success = a_kml_read_file ( vtl, f, external )) )
+        trw_layer_replace_external ( vtl, filename );
+      else
+        load_answer = LOAD_TYPE_KML_FAILURE;
+    }
     // NB use a extension check first, as a GPX file header may have a Byte Order Mark (BOM) in it
-    //    - which currently confuses our check_magic function
-    else if ( a_file_check_ext ( filename, ".gpx" ) || check_magic ( f, GPX_MAGIC, GPX_MAGIC_LEN ) ) {
-      if ( ! ( success = a_gpx_read_file ( vtl, f, dirpath ) ) ) {
-        load_answer = LOAD_TYPE_GPX_FAILURE;
+    //    - which currently confuses our file_check_magic function
+    else if ( a_file_check_ext ( filename, ".gpx" ) || file_check_magic ( f, FILE_XML_MAGIC ) ) {
+      GpxReadStatus_t read_status = a_gpx_read_file ( vtl, f, dirpath, !add_new );
+      switch (read_status) {
+      case GPX_READ_FAILURE: load_answer = LOAD_TYPE_GPX_FAILURE; break;
+      case GPX_READ_WARNING: load_answer = LOAD_TYPE_GPX_WARNING; break;
+      case GPX_READ_SUCCESS: load_answer = LOAD_TYPE_OTHER_SUCCESS; break;
       }
+      if ( load_answer != LOAD_TYPE_GPX_FAILURE ) {
+	if ( external )
+	  // TODO may have to make absolute??
+	  trw_layer_replace_external ( vtl, filename );
+      }
+    }
+    else if ( external && a_fit_check_magic ( f ) ) {
+      if ( (success = a_fit_read_file_into_layer ( vtl, f, filename )) )
+        trw_layer_replace_external ( vtl, filename );
+      else
+        load_answer = LOAD_TYPE_FIT_FAILURE;
     }
     else {
       // Try final supported file type
@@ -771,6 +854,45 @@ VikLoadType_t a_file_load ( VikAggregateLayer *top, VikViewport *vp, VikTrwLayer
       vik_trw_layer_auto_set_view ( vtl, vp );
     }
   }
+  return load_answer;
+}
+
+/**
+ * a_file_load:
+ *
+ */
+VikLoadType_t a_file_load ( VikAggregateLayer *top,
+                            VikViewport *vp,
+                            VikTrwLayer *vtl,
+                            const gchar *filename_or_uri,
+                            gboolean new_layer,
+                            gboolean external,
+                            const gchar *name )
+{
+  g_return_val_if_fail ( vp != NULL, LOAD_TYPE_READ_FAILURE );
+  g_return_val_if_fail ( filename_or_uri != NULL, LOAD_TYPE_READ_FAILURE );
+
+  char *filename = (char *)filename_or_uri;
+  if (strncmp(filename, "file://", 7) == 0) {
+    // Consider replacing this with:
+    // filename = g_filename_from_uri ( entry, NULL, NULL );
+    // Since this doesn't support URIs properly (i.e. will failure if is it has %20 characters in it)
+    filename = filename + 7;
+    g_debug ( "Loading file %s from URI %s", filename, filename_or_uri );
+  }
+  FILE *f = xfopen ( filename );
+
+  if ( ! f )
+    return LOAD_TYPE_READ_FAILURE;
+
+  gchar *absolute = file_realpath_dup ( filename );
+  gchar *dirpath = NULL;
+  if ( absolute )
+    dirpath = g_path_get_dirname ( absolute );
+  g_free ( absolute );
+
+  VikLoadType_t load_answer = a_file_load_stream ( f, filename, top, vp, vtl, new_layer, external, dirpath, name );
+
   g_free ( dirpath );
   xfclose(f);
   return load_answer;
@@ -815,7 +937,7 @@ gboolean a_file_save ( VikAggregateLayer *top, gpointer vp, const gchar *filenam
 }
 
 
-/* example: 
+/* example:
      gboolean is_gpx = a_file_check_ext ( "a/b/c.gpx", ".gpx" );
 */
 gboolean a_file_check_ext ( const gchar *filename, const gchar *fileext )
@@ -833,6 +955,21 @@ gboolean a_file_check_ext ( const gchar *filename, const gchar *fileext )
   return FALSE;
 }
 
+gboolean kml_export ( VikTrwLayer *vtl, const gchar *filename )
+{
+  gchar *ustr = NULL;
+  switch ( a_vik_get_kml_export_units () ) {
+  case VIK_KML_EXPORT_UNITS_STATUTE:  ustr = ""; break;
+  case VIK_KML_EXPORT_UNITS_NAUTICAL: ustr = ",units=n"; break;
+  default: ustr = ",units=m"; break;
+  }
+  gchar *babelargs = g_strdup_printf ("-o kml%s,track=%d,points=%d",
+                                      ustr, a_vik_get_kml_export_track(), a_vik_get_kml_export_points() );
+  gboolean ans = a_babel_convert_to ( vtl, NULL, babelargs, filename, NULL, NULL );
+  g_free ( babelargs );
+  return ans;
+}
+
 /**
  * a_file_export:
  * @vtl: The TrackWaypoint to export data from
@@ -846,7 +983,9 @@ gboolean a_file_check_ext ( const gchar *filename, const gchar *fileext )
  */
 gboolean a_file_export ( VikTrwLayer *vtl, const gchar *filename, VikFileType_t file_type, VikTrack *trk, gboolean write_hidden )
 {
-  GpxWritingOptions options = { FALSE, FALSE, write_hidden, FALSE };
+  g_return_val_if_fail ( vtl != NULL, FALSE );
+
+  GpxWritingOptions options = { FALSE, FALSE, write_hidden, FALSE, vik_trw_layer_get_gpx_version(vtl) };
   FILE *f = g_fopen ( filename, "w" );
   if ( f )
   {
@@ -858,7 +997,7 @@ gboolean a_file_export ( VikTrwLayer *vtl, const gchar *filename, VikFileType_t
         case FILE_TYPE_GPX:
           // trk defined so can set the option
           options.is_route = trk->is_route;
-          a_gpx_write_track_file ( trk, f, &options );
+          a_gpx_write_track_file ( vtl, trk, f, &options );
           break;
         default:
           g_critical("Houston, we've had a problem. file_type=%d", file_type);
@@ -878,20 +1017,9 @@ gboolean a_file_export ( VikTrwLayer *vtl, const gchar *filename, VikFileType_t
           result = a_geojson_write_file ( vtl, f );
           break;
         case FILE_TYPE_KML:
-	  fclose ( f );
-	  switch ( a_vik_get_kml_export_units () ) {
-	    case VIK_KML_EXPORT_UNITS_STATUTE:
-	      return a_babel_convert_to ( vtl, NULL, "-o kml", filename, NULL, NULL );
-	      break;
-	    case VIK_KML_EXPORT_UNITS_NAUTICAL:
-	      return a_babel_convert_to ( vtl, NULL, "-o kml,units=n", filename, NULL, NULL );
-	      break;
-	    default:
-	      // VIK_KML_EXPORT_UNITS_METRIC:
-	      return a_babel_convert_to ( vtl, NULL, "-o kml,units=m", filename, NULL, NULL );
-	      break;
-	  }
-	  break;
+          fclose ( f );
+          return kml_export ( vtl, filename );
+          break;
         default:
           g_critical("Houston, we've had a problem. file_type=%d", file_type);
       }
@@ -905,16 +1033,42 @@ gboolean a_file_export ( VikTrwLayer *vtl, const gchar *filename, VikFileType_t
 
 /**
  * a_file_export_babel:
+ * @vtl:         If specified, use this layer for the source of the export, otherwise use @gpxfilename for the source.
+ * @gpxfilename: If specified (and no @vtl), use this file as the source for the export.
+ * @filename:    The name of file that will be written to by the export. This must be specified.
+ * @format:      The GPSBabel format type of the export. This must be specified.
+ * @suboptions:  Further detailed command line options for the export. Optional.
+ *
+ * Returns if the export was successful or not
  */
-gboolean a_file_export_babel ( VikTrwLayer *vtl, const gchar *filename, const gchar *format,
-                               gboolean tracks, gboolean routes, gboolean waypoints )
+gboolean a_file_export_babel ( VikTrwLayer *vtl, const gchar *gpxfilename, const gchar *filename, const gchar *format,
+                               gboolean tracks, gboolean routes, gboolean waypoints, const gchar *suboptions )
 {
-  gchar *args = g_strdup_printf("%s %s %s -o %s",
+  g_return_val_if_fail ( filename != NULL, FALSE );
+  g_return_val_if_fail ( format != NULL, FALSE );
+
+  gchar *sopts = NULL;
+  if ( suboptions && strlen(suboptions) > 0 ) {
+    // Ensure there is a starting comma
+    if ( suboptions[0] != ',' )
+      sopts = g_strdup_printf ( ",%s", suboptions );
+    else
+      sopts = g_strdup ( ",%s" );
+  } else
+    sopts = g_strdup ( "" );
+
+  gchar *args = g_strdup_printf("%s %s %s -o %s%s",
                                 tracks ? "-t" : "",
                                 routes ? "-r" : "",
                                 waypoints ? "-w" : "",
-                                format);
-  gboolean result = a_babel_convert_to ( vtl, NULL, args, filename, NULL, NULL );
+                                format, sopts);
+  gboolean result = FALSE;
+  if ( vtl )
+    result = a_babel_convert_to ( vtl, NULL, args, filename, NULL, NULL );
+  else if ( gpxfilename )
+    result = a_babel_gpx_export ( args, gpxfilename, filename );
+
+  g_free(sopts);
   g_free(args);
   return result;
 }
