@@ -18,30 +18,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <errno.h>
-#include <time.h>
 
 #include <curl/curl.h>
 #include <curl/easy.h>
 
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-
 #include "viking.h"
 #include "viktrwlayer.h"
 #include "osm-traces.h"
 #include "gpx.h"
-#include "ui_util.h"
 #include "background.h"
-#include "preferences.h"
 #include "curl_download.h"
 #ifdef HAVE_OAUTH_H
 #include <oauth.h>
@@ -104,13 +90,17 @@ typedef struct _OsmTracesInfo {
   VikTrack *trk;
 } OsmTracesInfo;
 
+#ifdef VIK_CONFIG_OSM_AUTH
 static VikLayerParam prefs[] = {
+#ifdef HAVE_OAUTH_H
   { VIK_LAYER_NUM_TYPES, OSM_ACCESS_BUTTON, VIK_LAYER_PARAM_PTR, VIK_LAYER_GROUP_NONE, N_("Access Token:"), VIK_LAYER_WIDGET_BUTTON, N_("Get New OSM Access Tokens"), NULL, NULL, NULL, NULL, NULL },
   { VIK_LAYER_NUM_TYPES, OSM_ACCESS_TOKEN_KEY, VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("Access Token Key:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, NULL, NULL, NULL },
   { VIK_LAYER_NUM_TYPES, OSM_ACCESS_TOKEN_SECRET, VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("Access Token Secret:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, NULL, NULL, NULL },
+#endif
   { VIK_LAYER_NUM_TYPES, OSM_USERNAME, VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("OSM username:"), VIK_LAYER_WIDGET_ENTRY, NULL, NULL, NULL, NULL, NULL, NULL },
   { VIK_LAYER_NUM_TYPES, OSM_PASSWORD, VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("OSM password:"), VIK_LAYER_WIDGET_PASSWORD, NULL, NULL, NULL, NULL, NULL, NULL },
 };
+#endif
 
 /**
  * Free an OsmTracesInfo struct.
@@ -122,7 +112,6 @@ static void oti_free(OsmTracesInfo *oti)
     g_free(oti->name); oti->name = NULL;
     g_free(oti->description); oti->description = NULL;
     g_free(oti->tags); oti->tags = NULL;
-    
     g_object_unref(oti->vtl); oti->vtl = NULL;
   }
   /* Main struct has been g_malloc'ed */
@@ -167,7 +156,8 @@ gchar *osm_get_login()
 // https://wiki.openstreetmap.org/wiki/OAuth
 //
 static const gchar *viking_consumer_key = "bAUxFBhGSzwXo9R43gJ1JXqx8cVphItXLo0PsRV3";
-static const gchar *viking_consumer_secret = "DKWT1ydfZUF9VpiUVkMn3p7faIaUB2cBawh86a0z"; // Not exactly a secret as publically viewable right here!
+#define VSC_INIT "QXJG6lqsMHS4IcvHIxZa8c2snVnHO7pOnju31n5m"
+static gchar *viking_consumer_secret = NULL;
 
 /**
  * split and parse URL parameters replied by the server
@@ -291,7 +281,11 @@ static void new_access_token_cb ( )
   gchar *authorize_url = get_authorize_url ( request_key );
   g_debug ( "%s:%s", __FUNCTION__, authorize_url );
   if ( authorize_url ) {
-    gtk_show_uri ( gdk_screen_get_default(), authorize_url, GDK_CURRENT_TIME, NULL );
+#if GTK_CHECK_VERSION (3,22,0)
+    (void)gtk_show_uri_on_window ( parent, authorize_url, GDK_CURRENT_TIME, NULL );
+#else
+    (void)gtk_show_uri ( gdk_screen_get_default(), authorize_url, GDK_CURRENT_TIME, NULL );
+#endif
   }
   g_free ( authorize_url );
 
@@ -325,26 +319,17 @@ static void new_access_token_cb ( )
     return;
   }
 
-  // Unfortunately since the callback mechanism in preferences only allows a function call with no parameters
-  //  we have no way of accessing the dialog to effect any updates, other than closing it
-  a_uibuilder_factory_close ( GTK_RESPONSE_REJECT );
-  // Note since we are in an event handler already, using GTK_RESPONSE_ACCEPT means this signal
-  //  is processed after this current function finishes.
-  // This then in turn means it would save the values from the dialog,
-  //  overwriting the values set here :(
-  // The current side effect is that any other preferences the user has modified in the dialog are lost
-
   // Now apply the new values
   vlp_data.s = access_token_key;
   a_preferences_run_setparam ( vlp_data, pref_key );
   vlp_data.s = access_token_secret;
   a_preferences_run_setparam ( vlp_data, pref_secret );
 
-  a_preferences_save_to_file ();
-
   g_free ( access_token_key );
   g_free ( access_token_secret );
 
+  a_preferences_refresh ( VIKING_OSM_TRACES_PARAMS_GROUP_KEY );
+
   // On success mention can remove username/password if they already exist
   VikLayerParamData *pref_user = a_preferences_get ( OSM_USERNAME );
   VikLayerParamData *pref_pwd = a_preferences_get ( OSM_PASSWORD );
@@ -376,6 +361,7 @@ gchar* osm_oauth_sign_url ( const gchar* url, const char *method )
 /* initialisation */
 void osm_traces_init () {
   /* Preferences */
+#ifdef VIK_CONFIG_OSM_AUTH
   a_preferences_register_group ( VIKING_OSM_TRACES_PARAMS_GROUP_KEY, _("OpenStreetMap Traces") );
 
   guint ii = 0;
@@ -383,20 +369,22 @@ void osm_traces_init () {
 #ifdef HAVE_OAUTH_H
   tmp.ptr = new_access_token_cb;
   a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
-  tmp.s = "";
+  tmp.s = NULL;
   a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
-  tmp.s = "";
   a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
+  // Not that secret; but as least not in total plain sight anymore
+  viking_consumer_secret = g_strdup ( VSC_INIT );
+  viking_consumer_secret = util_frob ( viking_consumer_secret, strlen(viking_consumer_secret) );
 #endif
   // Only register depreciated preferences if they already exist
   //  or being forced to use the basic method
   if ( osm_use_basic_auth() ||
        (a_preferences_lookup("OSM_USERNAME") && a_preferences_lookup(OSM_PASSWORD)) ) {
-    tmp.s = "";
+    tmp.s = NULL;
     a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
-    tmp.s = "";
     a_preferences_register(&prefs[ii++], tmp, VIKING_OSM_TRACES_PARAMS_GROUP_KEY);
   }
+#endif
 
   login_mutex = vik_mutex_new();
 }
@@ -404,9 +392,12 @@ void osm_traces_init () {
 void osm_traces_uninit()
 {
   vik_mutex_free(login_mutex);
+#ifdef HAVE_OAUTH_H
+  g_free ( viking_consumer_secret );
+#endif
 }
 
-#define OSM_GPX_UPLOAD_URL "https://www.openstreetmap.org/api/0.6/gpx/create"
+#define OSM_GPX_UPLOAD_URL "https://api.openstreetmap.org/api/0.6/gpx/create"
 
 /*
  * Upload a file
@@ -516,7 +507,6 @@ static gint osm_traces_upload_file(const char *user,
   }
 
   /* Memory */
-  g_free(base_url);
   g_free(user_pass); user_pass = NULL;
   g_free(base_url);
 
@@ -526,17 +516,17 @@ static gint osm_traces_upload_file(const char *user,
 }
 
 /**
- * uploading function executed by the background" thread
+ * uploading function executed by the background thread
  */
 static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
 {
-  /* Due to OSM limits, we have to enforce ele and time fields
-   also don't upload invisible tracks */
-  static GpxWritingOptions options = { TRUE, TRUE, FALSE, FALSE };
-
   if (!oti)
     return;
 
+  /* Due to OSM limits, we have to enforce ele and time fields
+   also don't upload invisible tracks */
+  GpxWritingOptions options = { TRUE, TRUE, FALSE, FALSE, vik_trw_layer_get_gpx_version(oti->vtl) };
+
   gchar *filename = NULL;
 
   /* writing gpx file */
@@ -547,18 +537,18 @@ static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
     {
       VikTrack *trk = vik_track_copy(oti->trk, TRUE);
       vik_track_anonymize_times(trk);
-      filename = a_gpx_write_track_tmp_file(trk, &options);
+      filename = a_gpx_write_track_tmp_file (oti->vtl, trk, &options);
       vik_track_free(trk);
     }
     else
-      filename = a_gpx_write_track_tmp_file (oti->trk, &options);
+      filename = a_gpx_write_track_tmp_file (oti->vtl, oti->trk, &options);
   }
   else
   {
     /* Upload the whole VikTrwLayer */
     filename = a_gpx_write_tmp_file (oti->vtl, &options);
   }
-  
+
   if ( !filename )
     return;
 
@@ -579,7 +569,7 @@ static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
   timeinfo = localtime ( &timenow );
   gchar timestr[80];
   // Compact time only - as days/date isn't very useful here
-  strftime ( timestr, sizeof(timestr), "%X)", timeinfo );
+  strftime ( timestr, sizeof(timestr), "%X", timeinfo );
 
   //
   // Test to see if window it was invoked on is still valid
@@ -609,6 +599,7 @@ static void osm_traces_upload_thread ( OsmTracesInfo *oti, gpointer threaddata )
   if (ret != 0) {
     g_critical(_("failed to unlink temporary file: %s"), strerror(errno));
   }
+  g_free ( filename );
 }
 
 /**
@@ -622,7 +613,7 @@ void osm_login_widgets (GtkWidget *user_entry, GtkWidget *password_entry)
   const gchar *default_user = get_default_user();
   VikLayerParamData *pref_user = a_preferences_get ( OSM_USERNAME );
   VikLayerParamData *pref_password = a_preferences_get ( OSM_PASSWORD );
- 
+
   if (osm_user != NULL && osm_user[0] != '\0')
     gtk_entry_set_text(GTK_ENTRY(user_entry), osm_user);
   else if (pref_user && pref_user->s && pref_user->s[0] != '\0')
