@@ -19,22 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#ifdef HAVE_MATH_H
-#include <math.h>
-#endif
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gprintf.h>
-#include <glib/gi18n.h>
-
 #include "viking.h"
-
 #include "vikgotoxmltool.h"
 
 static void vik_goto_xml_tool_finalize ( GObject *gob );
@@ -43,6 +28,7 @@ static gchar *vik_goto_xml_tool_get_url_format ( VikGotoTool *self );
 static gboolean vik_goto_xml_tool_parse_file(VikGotoTool *self, gchar *filename);
 static gboolean vik_goto_xml_tool_parse_file_for_latlon(VikGotoTool *self, gchar *filename, struct LatLon *ll);
 static gboolean vik_goto_xml_tool_parse_file_for_candidates(VikGotoTool *self, gchar *filename, GList **candidates);
+static DownloadFileOptions *vik_goto_xml_tool_get_download_options(VikGotoTool *self);
 
 typedef struct _VikGotoXmlToolPrivate VikGotoXmlToolPrivate;
 
@@ -55,19 +41,18 @@ struct _VikGotoXmlToolPrivate
   gchar *lon_attr;
   gchar *desc_path;
   gchar *desc_attr;
-  
+
   struct LatLon ll;
   gchar *description;
 
   // if not null, load in all candidates
   GList **candidates;
+
+  DownloadFileOptions options;
 };
 
-#define GOTO_XML_TOOL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
-                                        VIK_GOTO_XML_TOOL_TYPE,          \
-                                        VikGotoXmlToolPrivate))
-
-G_DEFINE_TYPE (VikGotoXmlTool, vik_goto_xml_tool, VIK_GOTO_TOOL_TYPE)
+G_DEFINE_TYPE_WITH_PRIVATE (VikGotoXmlTool, vik_goto_xml_tool, VIK_GOTO_TOOL_TYPE)
+#define GOTO_XML_TOOL_GET_PRIVATE(o) (vik_goto_xml_tool_get_instance_private (VIK_GOTO_XML_TOOL(o)))
 
 enum
 {
@@ -80,6 +65,10 @@ enum
   PROP_LON_ATTR,
   PROP_DESC_PATH,
   PROP_DESC_ATTR,
+  PROP_REFERER,
+  PROP_USER_AGENT,
+  PROP_FOLLOW_LOCATION,
+  PROP_CUSTOM_HTTP_HEADERS,
 };
 
 static void
@@ -88,8 +77,7 @@ vik_goto_xml_tool_set_property (GObject      *object,
                                 const GValue *value,
                                 GParamSpec   *pspec)
 {
-  VikGotoXmlTool *self = VIK_GOTO_XML_TOOL (object);
-  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (self);
+  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (object);
   gchar **splitted = NULL;
 
   switch (property_id)
@@ -114,9 +102,6 @@ vik_goto_xml_tool_set_property (GObject      *object,
       break;
 
     case PROP_LAT_ATTR:
-      /* Avoid to overwrite XPATH value */
-      /* NB: This disable future overwriting,
-         but as property is CONSTRUCT_ONLY there is no matter */
       if (!priv->lat_attr || g_value_get_string (value))
       {
         g_free (priv->lat_attr);
@@ -139,9 +124,6 @@ vik_goto_xml_tool_set_property (GObject      *object,
       break;
 
     case PROP_LON_ATTR:
-      /* Avoid to overwrite XPATH value */
-      /* NB: This disable future overwriting,
-         but as property is CONSTRUCT_ONLY there is no matter */
       if (!priv->lon_attr || g_value_get_string (value))
       {
         g_free (priv->lon_attr);
@@ -174,6 +156,32 @@ vik_goto_xml_tool_set_property (GObject      *object,
       }
       break;
 
+    case PROP_REFERER:
+      g_free (priv->options.referer);
+      priv->options.referer = g_value_dup_string (value);
+      break;
+
+    case PROP_USER_AGENT:
+      g_free (priv->options.user_agent);
+      priv->options.user_agent = g_value_dup_string (value);
+      break;
+
+    case PROP_FOLLOW_LOCATION:
+      priv->options.follow_location = g_value_get_long (value);
+      break;
+
+    case PROP_CUSTOM_HTTP_HEADERS:
+      {
+      g_free (priv->options.custom_http_headers);
+      const gchar *str = g_value_get_string ( value );
+      // Convert the literal characters '\'+'n' into actual newlines '\n'
+      if ( str )
+        priv->options.custom_http_headers = g_strcompress ( str );
+      else
+        priv->options.custom_http_headers = NULL;
+      }
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -187,8 +195,7 @@ vik_goto_xml_tool_get_property (GObject    *object,
                                 GValue     *value,
                                 GParamSpec *pspec)
 {
-  VikGotoXmlTool *self = VIK_GOTO_XML_TOOL (object);
-  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (self);
+  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (object);
 
   switch (property_id)
     {
@@ -220,6 +227,22 @@ vik_goto_xml_tool_get_property (GObject    *object,
       g_value_set_string (value, priv->desc_attr);
       break;
 
+    case PROP_REFERER:
+      g_value_set_string (value, priv->options.referer);
+      break;
+
+    case PROP_USER_AGENT:
+      g_value_set_string (value, priv->options.user_agent);
+      break;
+
+    case PROP_FOLLOW_LOCATION:
+      g_value_set_long (value, priv->options.follow_location);
+      break;
+
+    case PROP_CUSTOM_HTTP_HEADERS:
+      g_value_set_string (value, priv->options.custom_http_headers);
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -259,11 +282,12 @@ vik_goto_xml_tool_class_init ( VikGotoXmlToolClass *klass )
                                    PROP_LAT_PATH,
                                    pspec);
 
+  // NB *not* CONSTRUCT_ONLY - to allow changing via XPath definition in lat-path
   pspec = g_param_spec_string ("lat-attr",
                                "Latitude attribute",
                                "XML attribute of the latitude",
                                NULL /* default value */,
-                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+                               G_PARAM_READWRITE);
   g_object_class_install_property (object_class,
                                    PROP_LAT_ATTR,
                                    pspec);
@@ -277,11 +301,12 @@ vik_goto_xml_tool_class_init ( VikGotoXmlToolClass *klass )
                                    PROP_LON_PATH,
                                    pspec);
 
+  // NB *not* CONSTRUCT_ONLY - to allow changing via XPath definition in lon-path
   pspec = g_param_spec_string ("lon-attr",
                                "Longitude attribute",
                                "XML attribute of the longitude",
                                NULL /* default value */,
-                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+                               G_PARAM_READWRITE);
   g_object_class_install_property (object_class,
                                    PROP_LON_ATTR,
                                    pspec);
@@ -304,19 +329,42 @@ vik_goto_xml_tool_class_init ( VikGotoXmlToolClass *klass )
                                    PROP_DESC_ATTR,
                                    pspec);
 
+  pspec = g_param_spec_string ("referer",
+                               "Referer",
+                               "The REFERER string to use in HTTP request",
+                               NULL /* default value */,
+                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_REFERER, pspec);
+
+  pspec = g_param_spec_string ("user-agent",
+                               "User Agent",
+                               "The User Agent string to send in the request",
+                               NULL, // default value
+                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_USER_AGENT, pspec);
+
+  pspec = g_param_spec_long ("follow-location",
+                             "Follow location",
+                             "Specifies the number of retries to follow a redirect while downloading a URL",
+                             -1, // minimum value (unlimited)
+                             G_MAXLONG, // maximum value
+                             -1, // default value - automatically follow location with unlimited redirects
+                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_FOLLOW_LOCATION, pspec);
+
+  pspec = g_param_spec_string ("custom-http-headers",
+                               "Custom HTTP Headers",
+                               "Custom HTTP Headers, use '\n' to separate multiple headers",
+                               NULL, // default value
+                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_CUSTOM_HTTP_HEADERS, pspec);
+
   parent_class = VIK_GOTO_TOOL_CLASS (klass);
 
   parent_class->get_url_format = vik_goto_xml_tool_get_url_format;
   parent_class->parse_file_for_latlon = vik_goto_xml_tool_parse_file_for_latlon;
   parent_class->parse_file_for_candidates = vik_goto_xml_tool_parse_file_for_candidates;
-
-  g_type_class_add_private (klass, sizeof (VikGotoXmlToolPrivate));
-}
-
-VikGotoXmlTool *
-vik_goto_xml_tool_new ()
-{
-  return VIK_GOTO_XML_TOOL ( g_object_new ( VIK_GOTO_XML_TOOL_TYPE, "label", "Google", NULL ) );
+  parent_class->get_download_options = vik_goto_xml_tool_get_download_options;
 }
 
 static void
@@ -330,17 +378,43 @@ vik_goto_xml_tool_init ( VikGotoXmlTool *self )
   priv->lon_attr = NULL;
   priv->desc_path = NULL;
   priv->desc_attr = NULL;
-  // 
+  //
   priv->ll.lat = NAN;
   priv->ll.lon = NAN;
   priv->description = NULL;
   priv->candidates = NULL;
+  // Options
+  priv->options.referer = NULL;
+  priv->options.follow_location = -1;
+  priv->options.expiry_age = 0;
+  priv->options.check_file = NULL;
+  priv->options.check_file_server_time = FALSE;
+  priv->options.use_etag = FALSE;
+  priv->options.user_agent = NULL;
+  priv->options.custom_http_headers = NULL;
 }
 
 static void
 vik_goto_xml_tool_finalize ( GObject *gob )
 {
-  G_OBJECT_GET_CLASS(gob)->finalize(gob);
+  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (gob);
+  g_free (priv->url_format);
+  g_free (priv->lat_path);
+  g_free (priv->lat_attr);
+  g_free (priv->lon_path);
+  g_free (priv->lon_attr);
+  g_free (priv->desc_path);
+  g_free (priv->desc_attr);
+  g_free (priv->description);
+  // IIUC description and candidates are merely here for convenience, with candidates being freed after use.
+  // Thus *priv->candidates will generally no longer be a valid pointer
+  //  so don't attempt to free it here.
+
+  g_free (priv->options.referer);
+  g_free (priv->options.user_agent);
+  g_free (priv->options.custom_http_headers);
+
+  G_OBJECT_CLASS (vik_goto_xml_tool_parent_class)->finalize(gob);
 }
 
 static gboolean
@@ -403,8 +477,7 @@ _start_element (GMarkupParseContext *context,
                 gpointer             user_data,
                 GError             **error)
 {
-  VikGotoXmlTool *self = VIK_GOTO_XML_TOOL (user_data);
-  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (self);
+  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (user_data);
   const GSList *stack = g_markup_parse_context_get_element_stack (context);
   /* Longitude */
   if (priv->lon_attr != NULL && isnan(priv->ll.lon) && stack_is_path (stack, priv->lon_path))
@@ -454,12 +527,11 @@ _start_element (GMarkupParseContext *context,
 static void
 _text (GMarkupParseContext *context,
        const gchar         *text,
-       gsize                text_len,  
+       gsize                text_len,
        gpointer             user_data,
        GError             **error)
 {
-  VikGotoXmlTool *self = VIK_GOTO_XML_TOOL (user_data);
-  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (self);
+  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (user_data);
   const GSList *stack = g_markup_parse_context_get_element_stack (context);
   gchar *textl = g_strndup(text, text_len);
   /* Store only first result */
@@ -484,7 +556,7 @@ _text (GMarkupParseContext *context,
 static gboolean
 vik_goto_xml_tool_parse_file(VikGotoTool *self, gchar *filename)
 {
-    g_debug("Parse %s\n", filename);
+	g_debug("Parse %s", filename);
 	GMarkupParser xml_parser;
 	GMarkupParseContext *xml_context = NULL;
 	GError *error = NULL;
@@ -501,7 +573,7 @@ vik_goto_xml_tool_parse_file(VikGotoTool *self, gchar *filename)
 	if (file == NULL)
 		/* TODO emit warning */
 		return FALSE;
-	
+
 	/* setup context parse (ie callbacks) */
 	if (priv->lat_attr != NULL || priv->lon_attr != NULL)
     // At least one coordinate uses an attribute
@@ -516,13 +588,13 @@ vik_goto_xml_tool_parse_file(VikGotoTool *self, gchar *filename)
     xml_parser.text = NULL;
 	xml_parser.passthrough = NULL;
 	xml_parser.error = NULL;
-	
+
 	xml_context = g_markup_parse_context_new(&xml_parser, 0, self, NULL);
 
 	/* setup result */
 	priv->ll.lat = NAN;
 	priv->ll.lon = NAN;
-	
+
 	gchar buff[BUFSIZ];
 	size_t nb;
 	while (xml_context &&
@@ -543,12 +615,17 @@ vik_goto_xml_tool_parse_file(VikGotoTool *self, gchar *filename)
 		fprintf(stderr, "%s: errors occurred while reading file: %s.\n",
 			__FUNCTION__, error->message);
 	g_clear_error (&error);
-	
+
 	if (xml_context)
 		g_markup_parse_context_free(xml_context);
 	xml_context = NULL;
 	fclose (file);
-  
+
+    // As candidates are individually prepended,
+    //  the list is reversed to preserve the ordering
+    if ( priv->candidates )
+      *priv->candidates = g_list_reverse ( *priv->candidates );
+
     return TRUE;
 }
 
@@ -564,7 +641,7 @@ vik_goto_xml_tool_parse_file_for_latlon(VikGotoTool *self, gchar *filename, stru
   {
     *ll = priv->ll;
   }
-  
+
   if (isnan(priv->ll.lat) || isnan(priv->ll.lat))
     /* At least one coordinate not found */
     return FALSE;
@@ -588,3 +665,11 @@ vik_goto_xml_tool_get_url_format ( VikGotoTool *self )
   g_return_val_if_fail(priv != NULL, NULL);
   return priv->url_format;
 }
+
+static DownloadFileOptions *
+vik_goto_xml_tool_get_download_options ( VikGotoTool *self )
+{
+  VikGotoXmlToolPrivate *priv = GOTO_XML_TOOL_GET_PRIVATE (self);
+  g_return_val_if_fail(priv != NULL, NULL);
+  return &(priv->options);
+}
