@@ -20,16 +20,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <glib/gi18n.h>
-
 #include "viking.h"
-#include <string.h>
-#include <stdlib.h>
 #include "viklayer_defaults.h"
 
 /* functions common to all layers. */
@@ -45,6 +36,9 @@ extern VikLayerInterface vik_dem_layer_interface;
 #ifdef HAVE_LIBMAPNIK
 extern VikLayerInterface vik_mapnik_layer_interface;
 #endif
+#ifdef HAVE_LIBGEOCLUE_2
+extern VikLayerInterface vik_geoclue_layer_interface;
+#endif
 
 enum {
   VL_UPDATE_SIGNAL,
@@ -71,7 +65,7 @@ static void vik_layer_class_init (VikLayerClass *klass)
   parent_class = g_type_class_peek_parent (klass);
 
   layer_signals[VL_UPDATE_SIGNAL] = g_signal_new ( "update", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (VikLayerClass, update), NULL, NULL, 
+      G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (VikLayerClass, update), NULL, NULL,
       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
   // Register all parameter defaults, early in the start up sequence
@@ -90,47 +84,6 @@ static gboolean idle_draw ( VikLayer *vl )
   return FALSE; // Nothing else to do
 }
 
-/**
- * Draw specified layer
- */
-void vik_layer_emit_update ( VikLayer *vl )
-{
-  if ( vl->visible && vl->realized ) {
-    GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
-    if ( !thread )
-      // Do nothing
-      return;
-
-    vik_window_set_redraw_trigger(vl);
-
-    // Only ever draw when there is time to do so
-    if ( g_thread_self() != thread )
-      // Drawing requested from another (background) thread, so handle via the gdk thread method
-      gdk_threads_add_idle ( (GSourceFunc) idle_draw, vl );
-    else
-      g_idle_add ( (GSourceFunc) idle_draw, vl );
-  }
-}
-
-/**
- * should only be done by VikLayersPanel (hence never used from the background)
- * need to redraw and record trigger when we make a layer invisible.
- */
-void vik_layer_emit_update_although_invisible ( VikLayer *vl )
-{
-  vik_window_set_redraw_trigger(vl);
-  g_idle_add ( (GSourceFunc) idle_draw, vl );
-}
-
-/* doesn't set the trigger. should be done by aggregate layer when child emits update. */
-void vik_layer_emit_update_secondary ( VikLayer *vl )
-{
-  if ( vl->visible )
-    // TODO: this can used from the background - eg in acquire
-    //       so will need to flow background update status through too
-    g_idle_add ( (GSourceFunc) idle_draw, vl );
-}
-
 static VikLayerInterface *vik_layer_interfaces[VIK_LAYER_NUM_TYPES] = {
   &vik_aggregate_layer_interface,
   &vik_trw_layer_interface,
@@ -142,8 +95,93 @@ static VikLayerInterface *vik_layer_interfaces[VIK_LAYER_NUM_TYPES] = {
 #ifdef HAVE_LIBMAPNIK
   &vik_mapnik_layer_interface,
 #endif
+#ifdef HAVE_LIBGEOCLUE_2
+  &vik_geoclue_layer_interface,
+#endif
 };
 
+/**
+ * Draw specified layer (c.f. vik_layer_draw)
+ * The draw is ensured to be in the GUI thread,
+ *  and happens at some point in the main loop update rather than 'now'
+ */
+void vik_layer_redraw ( VikLayer *vl )
+{
+  if ( vl->visible && vl->realized ) {
+    GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+    if ( !thread )
+      // Do nothing
+      return;
+
+    // Only ever draw when there is time to do so
+    if ( g_thread_self() != thread ) {
+      // Drawing requested from another (background) thread, so handle via the gdk thread method
+      (void)gdk_threads_add_idle ( (GSourceFunc)idle_draw, vl );
+    }
+    else {
+      (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+    }
+  }
+}
+
+/**
+ * An update event
+ * @is_modified: Whether the layer has been modified
+ * Redraw specified layer and a notification about the update
+ */
+void vik_layer_emit_update ( VikLayer *vl, gboolean is_modified )
+{
+  if ( vl->visible && vl->realized ) {
+    GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+    if ( !thread )
+      // Do nothing
+      return;
+
+    // Notionally the 'refresh' function could be directly connected to the 'update' signal
+    // However we then have to manage the lifecycle (creation, copying, removing, etc...)
+    //  ATM IMHO it's simpler to call/use the function here as we don't need
+    //  any of the features offered by the signal framework
+    // NB This is separate from the idle_draw as that is used by other kinds of updates as well
+
+    // Only ever draw when there is time to do so
+    if ( g_thread_self() != thread ) {
+      // Drawing requested from another (background) thread, so handle via the gdk thread method
+      (void)gdk_threads_add_idle ( (GSourceFunc)idle_draw, vl );
+      if ( vik_layer_interfaces[vl->type]->refresh )
+	(void)gdk_threads_add_idle ( (GSourceFunc)vik_layer_interfaces[vl->type]->refresh, vl );
+    }
+    else {
+      (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+      if ( vik_layer_interfaces[vl->type]->refresh )
+	(void)g_idle_add ( (GSourceFunc)vik_layer_interfaces[vl->type]->refresh, vl );
+    }
+  }
+
+  // Check for realized (similar to as above) is mainly to detect layers that in the process of being created
+  // As the widget doesn't exist yet, we can't get the owning vikwindow
+  // The modified status for new layers is generally handled in viklayerspanel instead.
+  if ( is_modified && vl->realized )
+    vik_window_set_modified ( (VikWindow *)(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+}
+
+/**
+ * should only be done by VikLayersPanel (hence never used from the background)
+ * need to redraw when we make a layer invisible.
+ */
+void vik_layer_emit_update_although_invisible ( VikLayer *vl )
+{
+  (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+}
+
+/* should be done by aggregate layer when child emits update. */
+void vik_layer_emit_update_secondary ( VikLayer *vl )
+{
+  if ( vl->visible )
+    // TODO: this can used from the background - eg in acquire
+    //       so will need to flow background update status through too
+    (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+}
+
 VikLayerInterface *vik_layer_get_interface ( VikLayerTypeEnum type )
 {
   g_assert ( type < VIK_LAYER_NUM_TYPES );
@@ -214,7 +252,14 @@ const gchar *vik_layer_get_name ( VikLayer *l )
   return l->name;
 }
 
-time_t vik_layer_get_timestamp ( VikLayer *vl )
+gchar *layer_get_new_name ( VikLayer *l )
+{
+  if ( vik_layer_interfaces[l->type]->get_new_name )
+    return vik_layer_interfaces[l->type]->get_new_name ( l );
+  return NULL;
+}
+
+gdouble vik_layer_get_timestamp ( VikLayer *vl )
 {
   if ( vik_layer_interfaces[vl->type]->get_timestamp )
     return vik_layer_interfaces[vl->type]->get_timestamp ( vl );
@@ -232,10 +277,17 @@ VikLayer *vik_layer_create ( VikLayerTypeEnum type, VikViewport *vp, gboolean in
 
   if ( interactive )
   {
-    if ( vik_layer_properties ( new_layer, vp, FALSE ) )
-      /* We translate the name here */
-      /* in order to avoid translating name set by user */
-      vik_layer_rename ( VIK_LAYER(new_layer), _(vik_layer_interfaces[type]->name) );
+    if ( vik_layer_properties ( new_layer, vp, FALSE ) ) {
+      gchar *new_name = layer_get_new_name ( new_layer );
+      if ( new_name ) {
+        vik_layer_rename ( VIK_LAYER(new_layer), new_name );
+        g_free ( new_name );
+      }
+      else
+        /* We translate the name here */
+        /* in order to avoid translating name set by user */
+        vik_layer_rename ( VIK_LAYER(new_layer), _(vik_layer_interfaces[type]->name) );
+    }
     else
     {
       g_object_unref ( G_OBJECT(new_layer) ); /* cancel that */
@@ -266,6 +318,13 @@ void vik_layer_draw ( VikLayer *l, VikViewport *vp )
       vik_layer_interfaces[l->type]->draw ( l, vp );
 }
 
+void vik_layer_configure ( VikLayer *l, VikViewport *vp )
+{
+  if ( l->visible )
+    if ( vik_layer_interfaces[l->type]->configure )
+      vik_layer_interfaces[l->type]->configure ( l, vp );
+}
+
 void vik_layer_change_coord_mode ( VikLayer *l, VikCoordMode mode )
 {
   if ( vik_layer_interfaces[l->type]->change_coord_mode )
@@ -337,7 +396,7 @@ void vik_layer_marshall_params ( VikLayer *vl, guint8 **data, guint *datalen )
       /* print out the string list in the array */
       case VIK_LAYER_PARAM_STRING_LIST: {
         GList *list = d.sl;
-        
+
         /* write length of list (# of strings) */
         guint listlen = g_list_length ( list );
         g_byte_array_append ( b, (guint8 *)&listlen, sizeof(listlen) );
@@ -357,7 +416,7 @@ void vik_layer_marshall_params ( VikLayer *vl, guint8 **data, guint *datalen )
       }
     }
   }
-  
+
   *data = b->data;
   *datalen = b->len;
   g_byte_array_free ( b, FALSE );
@@ -371,7 +430,7 @@ void vik_layer_unmarshall_params ( VikLayer *vl, const guint8 *data, guint datal
   VikLayerFuncSetParam set_param = vik_layer_get_interface(vl->type)->set_param;
   gchar *s;
   guint8 *b = (guint8 *)data;
-  
+
 #define vlm_size (*(gint *)b)
 #define vlm_read(obj)				\
   memcpy((obj), b+sizeof(guint), vlm_size);	\
@@ -398,7 +457,7 @@ void vik_layer_unmarshall_params ( VikLayer *vl, const guint8 *data, guint datal
       g_debug("%s: %s", __FUNCTION__, params[i].name);
       switch ( params[i].type )
       {
-      case VIK_LAYER_PARAM_STRING: 
+      case VIK_LAYER_PARAM_STRING:
 	s = g_malloc(vlm_size + 1);
 	s[vlm_size]=0;
 	vlm_read(s);
@@ -437,7 +496,7 @@ VikLayer *vik_layer_unmarshall ( const guint8 *data, guint len, VikViewport *vvp
 {
   header_t *header;
   header = (header_t *)data;
-  
+
   if ( vik_layer_interfaces[header->layer_type]->unmarshall ) {
     return vik_layer_interfaces[header->layer_type]->unmarshall ( header->data, header->len, vvp );
   } else {
@@ -468,7 +527,15 @@ gboolean vik_layer_selected ( VikLayer *l, gint subtype, gpointer sublayer, gint
   if ( vik_layer_interfaces[l->type]->layer_selected )
     return vik_layer_interfaces[l->type]->layer_selected ( l, subtype, sublayer, type, vlp );
   /* Since no 'layer_selected' function explicitly turn off here */
-  return vik_window_clear_highlight ( (VikWindow *)VIK_GTK_WINDOW_FROM_LAYER(l) );
+  return vik_window_clear_selected ( (VikWindow *)VIK_GTK_WINDOW_FROM_LAYER(l) );
+}
+
+// NB the layer->visible value has already been modified
+// This function acts to notify the layer of this event
+void vik_layer_layer_toggle_visible ( VikLayer *l )
+{
+  if ( vik_layer_interfaces[l->type]->layer_toggle_visible )
+    vik_layer_interfaces[l->type]->layer_toggle_visible ( l );
 }
 
 void vik_layer_realize ( VikLayer *l, VikTreeview *vt, GtkTreeIter *layer_iter )
@@ -480,13 +547,13 @@ void vik_layer_realize ( VikLayer *l, VikTreeview *vt, GtkTreeIter *layer_iter )
     vik_layer_interfaces[l->type]->realize ( l, vt, layer_iter );
 }
 
-void vik_layer_set_menu_items_selection(VikLayer *l, guint16 selection)
+void vik_layer_set_menu_items_selection(VikLayer *l, VikStdLayerMenuItem selection)
 {
   if ( vik_layer_interfaces[l->type]->set_menu_selection )
     vik_layer_interfaces[l->type]->set_menu_selection ( l, selection );
 }
 
-guint16 vik_layer_get_menu_items_selection(VikLayer *l)
+VikStdLayerMenuItem vik_layer_get_menu_items_selection(VikLayer *l)
 {
   if ( vik_layer_interfaces[l->type]->get_menu_selection )
     return(vik_layer_interfaces[l->type]->get_menu_selection (l));
@@ -494,16 +561,16 @@ guint16 vik_layer_get_menu_items_selection(VikLayer *l)
     return(vik_layer_interfaces[l->type]->menu_items_selection);
 }
 
-void vik_layer_add_menu_items ( VikLayer *l, GtkMenu *menu, gpointer vlp )
+void vik_layer_add_menu_items ( VikLayer *l, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter )
 {
   if ( vik_layer_interfaces[l->type]->add_menu_items )
-    vik_layer_interfaces[l->type]->add_menu_items ( l, menu, vlp );
+    vik_layer_interfaces[l->type]->add_menu_items ( l, menu, vlp, selection, iter );
 }
 
-gboolean vik_layer_sublayer_add_menu_items ( VikLayer *l, GtkMenu *menu, gpointer vlp, gint subtype, gpointer sublayer, GtkTreeIter *iter, VikViewport *vvp )
+gboolean vik_layer_sublayer_add_menu_items ( VikLayer *l, GtkMenu *menu, gpointer vlp, gint subtype, gpointer sublayer, GtkTreeIter *iter, VikViewport *vvp, VikStdLayerMenuItem selection )
 {
   if ( vik_layer_interfaces[l->type]->sublayer_add_menu_items )
-    return vik_layer_interfaces[l->type]->sublayer_add_menu_items ( l, menu, vlp, subtype, sublayer, iter, vvp );
+    return vik_layer_interfaces[l->type]->sublayer_add_menu_items ( l, menu, vlp, subtype, sublayer, iter, vvp, selection );
   return FALSE;
 }
 
@@ -529,19 +596,30 @@ const gchar* vik_layer_layer_tooltip ( VikLayer *l )
   return NULL;
 }
 
+/**
+ * vik_layer_load_icon:
+ *
+ * Unref the returned pixbuf after use
+ */
 GdkPixbuf *vik_layer_load_icon ( VikLayerTypeEnum type )
 {
   g_assert ( type < VIK_LAYER_NUM_TYPES );
   if ( vik_layer_interfaces[type]->icon )
-    return gdk_pixbuf_from_pixdata ( vik_layer_interfaces[type]->icon, FALSE, NULL );
+    return ui_get_icon ( vik_layer_interfaces[type]->icon, 16 );
   return NULL;
 }
 
 gboolean vik_layer_set_param ( VikLayer *vl, VikLayerSetParam *vlsp )
 {
+  gboolean ans = FALSE;
   if ( vik_layer_interfaces[vl->type]->set_param )
-    return vik_layer_interfaces[vl->type]->set_param ( vl, vlsp );
-  return FALSE;
+    ans = vik_layer_interfaces[vl->type]->set_param ( vl, vlsp );
+  if ( ans && !vlsp->is_file_operation && vl->realized ) {
+    VikWindow *vw = (VikWindow*)VIK_GTK_WINDOW_FROM_LAYER(vl);
+    if ( vw )
+      vik_window_set_modified ( vw );
+  }
+  return ans;
 }
 
 void vik_layer_post_read ( VikLayer *layer, VikViewport *vp, gboolean from_file )
@@ -550,10 +628,9 @@ void vik_layer_post_read ( VikLayer *layer, VikViewport *vp, gboolean from_file
     vik_layer_interfaces[layer->type]->post_read ( layer, vp, from_file );
 }
 
-// Wrapper function to keep compiler happy
 static void layer_emit_update_internal ( gpointer gp )
 {
-  vik_layer_emit_update ( VIK_LAYER(gp));
+  vik_layer_emit_update ( VIK_LAYER(gp), TRUE );
 }
 
 static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gboolean have_apply )
@@ -562,23 +639,25 @@ static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gb
 					    VIK_GTK_WINDOW_FROM_WIDGET(vp),
 					    vik_layer_interfaces[vl->type]->params,
 					    vik_layer_interfaces[vl->type]->params_count,
+					    vik_layer_interfaces[vl->type]->params_offset,
 					    vik_layer_interfaces[vl->type]->params_groups,
 					    vik_layer_interfaces[vl->type]->params_groups_count,
-					    (gpointer) vik_layer_interfaces[vl->type]->set_param,
+                                            vik_layer_interfaces[vl->type]->default_group ? vik_layer_interfaces[vl->type]->default_group(vl) : 0,
+					    (gpointer) vik_layer_set_param,
 					    NULL,
-					    vl, 
+					    vl,
 					    vp,
-					    (gpointer) vik_layer_interfaces[vl->type]->get_param, 
+					    (gpointer) vik_layer_interfaces[vl->type]->get_param,
 					    vl,
 					    (gpointer) vik_layer_interfaces[vl->type]->change_param,
 					    have_apply,
 					    layer_emit_update_internal,
-					    (gpointer)vl) ) {
+					    (gpointer)vl,
+					    have_apply) ) {
     case 0:
-    case 3:
       return FALSE;
-      /* redraw (?) */
     case 2:
+    case 3:
       vik_layer_post_read ( vl, vp, FALSE ); /* update any gc's */
     default:
       return TRUE;
@@ -691,7 +770,11 @@ void vik_layer_set_defaults ( VikLayer *vl, VikViewport *vvp )
       if ( vli->params[i].type != VIK_LAYER_PARAM_STRING_LIST ) {
         vlsp.data = a_layer_defaults_get ( layer_name, vli->params[i].name, vli->params[i].type );
         (void)vik_layer_set_param ( vl, &vlsp );
+        if ( vli->params[i].type == VIK_LAYER_PARAM_STRING )
+          g_free ( (gchar*)vlsp.data.s );
       }
+      else
+        g_debug ( "%s: No files list default", __FUNCTION__ );
     }
   }
 }
@@ -703,3 +786,60 @@ void vik_layer_expand_tree ( VikLayer *vl )
 {
   vik_treeview_expand ( vl->vt, &vl->iter );
 }
+
+/**
+ * Returns whether the new value is an actual change
+ */
+gboolean vik_layer_param_change_boolean ( VikLayerParamData val, gboolean *current )
+{
+  gboolean old = *current;
+  *current = val.b;
+  return (old != val.b);
+}
+
+gboolean vik_layer_param_change_uint ( VikLayerParamData val, guint *current )
+{
+  guint old = *current;
+  *current = val.u;
+  return (old != val.u);
+}
+
+gboolean vik_layer_param_change_uint8 ( VikLayerParamData val, guint8 *current )
+{
+  guint8 old = *current;
+  *current = val.u;
+  return (old != val.u);
+}
+
+gboolean vik_layer_param_change_int ( VikLayerParamData val, gint *current )
+{
+  gint old = *current;
+  *current = val.i;
+  return (old != val.i);
+}
+
+gboolean vik_layer_param_change_color ( VikLayerParamData val, GdkColor *current )
+{
+  GdkColor old = *current;
+  *current = val.c;
+  return !gdk_color_equal ( &old, &val.c );
+}
+
+// Doubles may be in some kind of units - metres/feet etc...
+//  and in that case should not use this function directly
+gboolean vik_layer_param_change_double ( VikLayerParamData val, gdouble *current )
+{
+  gdouble old = *current;
+  *current = val.d;
+  return util_gdouble_different ( old, val.d );
+}
+
+gboolean vik_layer_param_change_string ( VikLayerParamData val, gchar **current )
+{
+  gchar* old = g_strdup ( *current );
+  g_free ( *current );
+  *current = g_strdup ( val.s );
+  gboolean changed = g_strcmp0 ( old, val.s );
+  g_free ( old );
+  return changed;
+}
