@@ -18,18 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "viking.h"
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-#include <string.h>
-#include <math.h>
-#include <stdlib.h>
+#include "vikmapniklayer.h"
 #include <ctype.h>
 
 #include "map_ids.h"
@@ -37,15 +27,13 @@
 #include "mapcoord.h"
 #include "mapcache.h"
 #include "dir.h"
-#include "util.h"
-#include "ui_util.h"
-#include "preferences.h"
-#include "icons/icons.h"
 #include "mapnik_interface.h"
 #include "background.h"
 
 #include "vikmapslayer.h"
 
+#define MAPNIK_FIXED_NAME "Mapnik Rendering"
+
 struct _VikMapnikLayerClass
 {
 	VikLayerClass object_class;
@@ -74,6 +62,13 @@ static VikLayerParamScale scales[] = {
 	{ 0, 1024, 12, 0 }, // Rerender timeout hours
 };
 
+static void reset_cb ( GtkWidget *widget, gpointer ptr )
+{
+	a_layer_defaults_reset_show ( MAPNIK_FIXED_NAME, ptr, VIK_LAYER_GROUP_NONE );
+}
+
+static VikLayerParamData reset_default ( void ) { return VIK_LPD_PTR(reset_cb); }
+
 VikLayerParam mapnik_layer_params[] = {
   { VIK_LAYER_MAPNIK, "config-file-mml", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("CSS (MML) Config File:"), VIK_LAYER_WIDGET_FILEENTRY, GINT_TO_POINTER(VF_FILTER_CARTO), NULL,
     N_("CartoCSS configuration file"), file_default, NULL, NULL },
@@ -85,6 +80,8 @@ VikLayerParam mapnik_layer_params[] = {
     NULL, vik_lpd_true_default, NULL, NULL },
   { VIK_LAYER_MAPNIK, "file-cache-dir", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("File Cache Directory:"), VIK_LAYER_WIDGET_FOLDERENTRY, NULL, NULL,
     NULL, cache_dir_default, NULL, NULL },
+  { VIK_LAYER_MAPNIK, "reset", VIK_LAYER_PARAM_PTR_DEFAULT, VIK_LAYER_GROUP_NONE, NULL,
+    VIK_LAYER_WIDGET_BUTTON, N_("Reset to Defaults"), NULL, NULL, reset_default, NULL, NULL },
 };
 
 enum {
@@ -93,25 +90,26 @@ enum {
   PARAM_ALPHA,
   PARAM_USE_FILE_CACHE,
   PARAM_FILE_CACHE_DIR,
+  PARAM_RESET,
   NUM_PARAMS };
 
 static const gchar* mapnik_layer_tooltip ( VikMapnikLayer *vml );
 static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, guint *len );
 static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp );
-static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation );
+static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, VikLayerSetParam *vlsp );
 static VikLayerParamData mapnik_layer_get_param ( VikMapnikLayer *vml, guint16 id, gboolean is_file_operation );
 static VikMapnikLayer *mapnik_layer_new ( VikViewport *vvp );
 static VikMapnikLayer *mapnik_layer_create ( VikViewport *vp );
 static void mapnik_layer_free ( VikMapnikLayer *vml );
 static void mapnik_layer_draw ( VikMapnikLayer *vml, VikViewport *vp );
-static void mapnik_layer_add_menu_items ( VikMapnikLayer *vml, GtkMenu *menu, gpointer vlp );
+static void mapnik_layer_add_menu_items ( VikMapnikLayer *vml, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter );
 
 static gpointer mapnik_feature_create ( VikWindow *vw, VikViewport *vvp)
 {
   return vvp;
 }
 
-static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp );
+static VikLayerToolFuncStatus mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp );
 
 // See comment in viktrwlayer.c for advice on values used
 // FUTURE:
@@ -136,28 +134,31 @@ static VikToolInterface mapnik_tools[] = {
 static void mapnik_layer_post_read (VikLayer *vl, VikViewport *vvp, gboolean from_file);
 
 VikLayerInterface vik_mapnik_layer_interface = {
-	"Mapnik Rendering",
+	MAPNIK_FIXED_NAME,
 	N_("Mapnik Rendering"),
 	NULL,
-	&vikmapniklayer_pixbuf, // icon
+	"vikmapniklayer", // Icon name
 
 	mapnik_tools,
-	sizeof(mapnik_tools) / sizeof(VikToolInterface),
+	G_N_ELEMENTS(mapnik_tools),
 
 	mapnik_layer_params,
 	NUM_PARAMS,
+	0, // Number of VIK_LAYER_NOT_IN_PROPERTIES
 	NULL,
 	0,
 
 	VIK_MENU_ITEM_ALL,
 
 	(VikLayerFuncCreate)                  mapnik_layer_create,
+	(VikLayerFuncGetNewName)              NULL,
 	(VikLayerFuncRealize)                 NULL,
 	(VikLayerFuncPostRead)                mapnik_layer_post_read,
 	(VikLayerFuncFree)                    mapnik_layer_free,
 
 	(VikLayerFuncProperties)              NULL,
 	(VikLayerFuncDraw)                    mapnik_layer_draw,
+	(VikLayerFuncConfigure)               NULL,
 	(VikLayerFuncChangeCoordMode)         NULL,
 
 	(VikLayerFuncGetTimestamp)            NULL,
@@ -173,10 +174,12 @@ VikLayerInterface vik_mapnik_layer_interface = {
 	(VikLayerFuncSublayerTooltip)         NULL,
 	(VikLayerFuncLayerTooltip)            mapnik_layer_tooltip,
 	(VikLayerFuncLayerSelected)           NULL,
+	(VikLayerFuncLayerToggleVisible)      NULL,
 
 	(VikLayerFuncMarshall)                mapnik_layer_marshall,
 	(VikLayerFuncUnmarshall)              mapnik_layer_unmarshall,
 
+	(VikLayerFuncParamsDefaultGroup)      NULL,
 	(VikLayerFuncSetParam)                mapnik_layer_set_param,
 	(VikLayerFuncGetParam)                mapnik_layer_get_param,
 	(VikLayerFuncChangeParam)             NULL,
@@ -195,6 +198,8 @@ VikLayerInterface vik_mapnik_layer_interface = {
 	(VikLayerFuncSelectMove)              NULL,
 	(VikLayerFuncSelectRelease)           NULL,
 	(VikLayerFuncSelectedViewportMenu)    NULL,
+
+	(VikLayerFuncRefresh)                 NULL,
 };
 
 struct _VikMapnikLayer {
@@ -228,7 +233,12 @@ static VikLayerParamData plugins_default ( void )
 #else
 	if ( g_file_test ( "/usr/lib/mapnik/input", G_FILE_TEST_EXISTS ) )
 		data.s = g_strdup ( "/usr/lib/mapnik/input" );
-		// Current Debian locations
+	// Current Debian locations
+	//  only try main stream AMD64 architecture
+	else if ( g_file_test ( "/usr/lib/x86_64-linux-gnu/mapnik/4.0/input", G_FILE_TEST_EXISTS ) )
+		data.s = g_strdup ( "/usr/lib/x86_64-linux-gnu/mapnik/4.0/input" );
+	else if ( g_file_test ( "/usr/lib/mapnik/3.1/input", G_FILE_TEST_EXISTS ) )
+		data.s = g_strdup ( "/usr/lib/mapnik/3.1/input" );
 	else if ( g_file_test ( "/usr/lib/mapnik/3.0/input", G_FILE_TEST_EXISTS ) )
 		data.s = g_strdup ( "/usr/lib/mapnik/3.0/input" );
 	else if ( g_file_test ( "/usr/lib/mapnik/2.2/input", G_FILE_TEST_EXISTS ) )
@@ -253,14 +263,22 @@ static VikLayerParamData fonts_default ( void )
 	return data;
 }
 
+static VikLayerParamData carto_default ( void ) {
+  VikLayerParamData data;
+  data.s = g_strdup ( "carto" );
+  return data;
+}
+
+static VikLayerParamData rr_to_default ( void ) { return VIK_LPD_UINT(168); } // One week in hours
+
 static VikLayerParam prefs[] = {
 	// Changing these values only applies before first mapnik layer is 'created'
 	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"plugins_directory", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("Plugins Directory:"), VIK_LAYER_WIDGET_FOLDERENTRY, NULL, NULL, N_("You need to restart Viking for a change to this value to be used"), plugins_default, NULL, NULL },
 	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"fonts_directory", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("Fonts Directory:"), VIK_LAYER_WIDGET_FOLDERENTRY, NULL, NULL, N_("You need to restart Viking for a change to this value to be used"), fonts_default, NULL, NULL },
 	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"recurse_fonts_directory", VIK_LAYER_PARAM_BOOLEAN, VIK_LAYER_GROUP_NONE, N_("Recurse Fonts Directory:"), VIK_LAYER_WIDGET_CHECKBUTTON, NULL, NULL, N_("You need to restart Viking for a change to this value to be used"), vik_lpd_true_default, NULL, NULL },
-	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"rerender_after", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Rerender Timeout (hours):"), VIK_LAYER_WIDGET_SPINBUTTON, &scales[2], NULL, N_("You need to restart Viking for a change to this value to be used"), NULL, NULL, NULL },
+	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"rerender_after", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Rerender Timeout (hours):"), VIK_LAYER_WIDGET_SPINBUTTON, &scales[2], NULL, N_("You need to restart Viking for a change to this value to be used"), rr_to_default, NULL, NULL },
 	// Changeable any time
-	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"carto", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("CartoCSS:"), VIK_LAYER_WIDGET_FILEENTRY, NULL, NULL,  N_("The program to convert CartoCSS files into Mapnik XML"), NULL, NULL, NULL },
+	{ VIK_LAYER_NUM_TYPES, MAPNIK_PREFS_NAMESPACE"carto", VIK_LAYER_PARAM_STRING, VIK_LAYER_GROUP_NONE, N_("CartoCSS:"), VIK_LAYER_WIDGET_FILEENTRY, NULL, NULL,  N_("The program to convert CartoCSS files into Mapnik XML"), carto_default, NULL, NULL },
 };
 
 static time_t planet_import_time;
@@ -268,6 +286,8 @@ static time_t planet_import_time;
 static GMutex *tp_mutex;
 static GHashTable *requests = NULL;
 
+static GdkColor black_color;
+
 /**
  * vik_mapnik_layer_init:
  *
@@ -277,21 +297,10 @@ void vik_mapnik_layer_init (void)
 {
 	a_preferences_register_group ( MAPNIK_PREFS_GROUP_KEY, _("Mapnik") );
 
-	guint i = 0;
-	VikLayerParamData tmp = plugins_default();
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
+	for ( guint ii = 0; ii < G_N_ELEMENTS(prefs); ii++ )
+	  a_preferences_register ( &prefs[ii], (VikLayerParamData){0}, MAPNIK_PREFS_GROUP_KEY );
 
-	tmp = fonts_default();
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
-
-	tmp.b = TRUE;
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
-
-	tmp.u = 168; // One week
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
-
-	tmp.s = "carto";
-	a_preferences_register(&prefs[i++], tmp, MAPNIK_PREFS_GROUP_KEY);
+        gdk_color_parse ( "#000000", &black_color );
 }
 
 /**
@@ -327,6 +336,7 @@ void vik_mapnik_layer_post_init (void)
 void vik_mapnik_layer_uninit ()
 {
 	vik_mutex_free (tp_mutex);
+	g_hash_table_destroy ( requests );
 }
 
 // NB Only performed once per program run
@@ -380,20 +390,6 @@ static void mapnik_layer_set_file_xml ( VikMapnikLayer *vml, const gchar *name )
 		vml->filename_xml = g_strdup (name);
 }
 
-static void mapnik_layer_set_file_css ( VikMapnikLayer *vml, const gchar *name )
-{
-	if ( vml->filename_css )
-		g_free (vml->filename_css);
-	vml->filename_css = g_strdup (name);
-}
-
-static void mapnik_layer_set_cache_dir ( VikMapnikLayer *vml, const gchar *name )
-{
-	if ( vml->file_cache_dir )
-		g_free (vml->file_cache_dir);
-	vml->file_cache_dir = g_strdup (name);
-}
-
 static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, guint *len )
 {
 	vik_layer_marshall_params ( VIK_LAYER(vml), data, len );
@@ -406,16 +402,34 @@ static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, guint len, VikView
 	return rv;
 }
 
-static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation )
+static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, VikLayerSetParam *vlsp )
 {
-	switch ( id ) {
-		case PARAM_CONFIG_CSS: mapnik_layer_set_file_css (vml, data.s); break;
-		case PARAM_CONFIG_XML: mapnik_layer_set_file_xml (vml, data.s); break;
-		case PARAM_ALPHA: if ( data.u <= 255 ) vml->alpha = data.u; break;
-		case PARAM_USE_FILE_CACHE: vml->use_file_cache = data.b; break;
-		case PARAM_FILE_CACHE_DIR: mapnik_layer_set_cache_dir (vml, data.s); break;
+	gboolean changed = FALSE;
+	switch ( vlsp->id ) {
+		case PARAM_CONFIG_CSS:
+			changed = vik_layer_param_change_string ( vlsp->data, &vml->filename_css );
+			break;
+		case PARAM_CONFIG_XML: {
+			gchar* old = g_strdup ( vml->filename_xml );
+			mapnik_layer_set_file_xml ( vml, vlsp->data.s );
+			changed = g_strcmp0 ( old, vlsp->data.s );
+			g_free ( old );
+			break;
+		}
+		case PARAM_ALPHA:
+			if ( vlsp->data.u <= 255 )
+				changed = vik_layer_param_change_uint8 ( vlsp->data, &vml->alpha );
+			break;
+		case PARAM_USE_FILE_CACHE:
+			changed = vik_layer_param_change_boolean ( vlsp->data, &vml->use_file_cache );
+			break;
+		case PARAM_FILE_CACHE_DIR:
+			changed = vik_layer_param_change_string ( vlsp->data, &vml->file_cache_dir );
+			break;
 		default: break;
 	}
+	if ( vik_debug && changed )
+		g_debug ( "%s: Detected change on param %d", __FUNCTION__, vlsp->id );
 	return TRUE;
 }
 
@@ -460,6 +474,7 @@ static VikLayerParamData mapnik_layer_get_param ( VikMapnikLayer *vml, guint16 i
 		case PARAM_ALPHA: data.u = vml->alpha; break;
 		case PARAM_USE_FILE_CACHE: data.b = vml->use_file_cache; break;
 		case PARAM_FILE_CACHE_DIR: data.s = vml->file_cache_dir; break;
+		case PARAM_RESET: data.ptr = reset_cb; break;
 		default: break;
 	}
 	return data;
@@ -509,17 +524,11 @@ gboolean carto_load ( VikMapnikLayer *vml, VikViewport *vvp )
 		g_free ( msg );
 	}
 
-	gint64 tt1 = 0;
+	gint64 tt1 = g_get_real_time ();
 	gint64 tt2 = 0;
-	// You won't get a sensible timing measurement if running too old a GLIB
-#if GLIB_CHECK_VERSION (2, 28, 0)
-	tt1 = g_get_real_time ();
-#endif
 
 	if ( g_spawn_command_line_sync ( command, &mystdout, &mystderr, NULL, &error ) ) {
-#if GLIB_CHECK_VERSION (2, 28, 0)
 		tt2 = g_get_real_time ();
-#endif
 		if ( mystderr )
 			if ( strlen(mystderr) > 1 ) {
 				a_dialog_error_msg_extra ( VIK_GTK_WINDOW_FROM_WIDGET(vvp), _("Error running carto command:\n%s"), mystderr );
@@ -672,14 +681,14 @@ static void render ( VikMapnikLayer *vml, VikCoord *ul, VikCoord *br, MapCoord *
 	g_debug ( "Mapnik rendering completed in %.3f seconds", tt );
 	if ( !pixbuf ) {
 		// A pixbuf to stick into cache incase of an unrenderable area - otherwise will get continually re-requested
-		pixbuf = gdk_pixbuf_scale_simple ( gdk_pixbuf_from_pixdata(&vikmapniklayer_pixbuf, FALSE, NULL), vml->tile_size_x, vml->tile_size_x, GDK_INTERP_BILINEAR );
+		pixbuf = gdk_pixbuf_scale_simple ( ui_get_icon("vikmapniklayer", 16), vml->tile_size_x, vml->tile_size_x, GDK_INTERP_BILINEAR );
 	}
 	possibly_save_pixbuf ( vml, pixbuf, ulm );
 
 	// NB Mapnik can apply alpha, but use our own function for now
 	if ( vml->alpha < 255 )
 		pixbuf = ui_pixbuf_scale_alpha ( pixbuf, vml->alpha );
-	a_mapcache_add ( pixbuf, (mapcache_extra_t){ tt }, ulm->x, ulm->y, ulm->z, MAP_ID_MAPNIK_RENDER, ulm->scale, vml->alpha, 0.0, 0.0, vml->filename_xml );
+	a_mapcache_add ( pixbuf, (mapcache_extra_t){ tt, 0 }, ulm->x, ulm->y, ulm->z, MAP_ID_MAPNIK_RENDER, ulm->scale, vml->alpha, 0.0, 0.0, vml->filename_xml );
 	g_object_unref(pixbuf);
 }
 
@@ -704,7 +713,7 @@ static void background ( RenderInfo *data, gpointer threaddata )
 	g_mutex_unlock(tp_mutex);
 
 	if (res == 0)
-		vik_layer_emit_update ( VIK_LAYER(data->vml) ); // NB update display from background
+		vik_layer_emit_update ( VIK_LAYER(data->vml), FALSE ); // NB update display from background
 }
 
 static void render_cancel_cleanup (RenderInfo *data)
@@ -819,7 +828,7 @@ static GdkPixbuf *get_pixbuf ( VikMapnikLayer *vml, MapCoord *ulm, MapCoord *brm
 			else {
 				// Run in the foreground
 				render ( vml, &ul, &br, ulm );
-				vik_layer_emit_update ( VIK_LAYER(vml) );
+				vik_layer_emit_update ( VIK_LAYER(vml), FALSE );
 			}
 		}
 	}
@@ -892,7 +901,7 @@ static void mapnik_layer_draw ( VikMapnikLayer *vml, VikViewport *vvp )
 		// Done after so drawn on top
 		// Just a handy guide to tile blocks.
 		if ( vik_debug && vik_verbose ) {
-			GdkGC *black_gc = GTK_WIDGET(vvp)->style->black_gc;
+			GdkGC *black_gc = vik_viewport_get_black_gc ( vvp );
 			gint width = vik_viewport_get_width(vvp);
 			gint height = vik_viewport_get_height(vvp);
 			gint xx, yy;
@@ -902,11 +911,11 @@ static void mapnik_layer_draw ( VikMapnikLayer *vml, VikViewport *vvp )
 			xx = xx - (vml->tile_size_x/2);
 			yy = yy - (vml->tile_size_x/2); // Yes use X ATM
 			for (gint x = xmin; x <= xmax; x++ ) {
-				vik_viewport_draw_line ( vvp, black_gc, xx, 0, xx, height );
+				vik_viewport_draw_line ( vvp, black_gc, xx, 0, xx, height, &black_color, 1 );
 				xx += vml->tile_size_x;
 			}
 			for (gint y = ymin; y <= ymax; y++ ) {
-				vik_viewport_draw_line ( vvp, black_gc, 0, yy, width, yy );
+				vik_viewport_draw_line ( vvp, black_gc, 0, yy, width, yy, &black_color, 1 );
 				yy += vml->tile_size_x; // Yes use X ATM
 			}
 		}
@@ -1016,47 +1025,27 @@ static void mapnik_layer_about ( menu_array_values values )
 /**
  *
  */
-static void mapnik_layer_add_menu_items ( VikMapnikLayer *vml, GtkMenu *menu, gpointer vlp )
+static void mapnik_layer_add_menu_items ( VikMapnikLayer *vml, GtkMenu *menu, gpointer vlp, VikStdLayerMenuItem selection, GtkTreeIter *iter )
 {
 	static menu_array_values values;
 	values[MA_VML] = vml;
 	values[MA_VVP] = vik_layers_panel_get_viewport( VIK_LAYERS_PANEL(vlp) );
 
-	GtkWidget *item = gtk_menu_item_new();
-	gtk_menu_shell_append ( GTK_MENU_SHELL(menu), item );
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, NULL, NULL, NULL, NULL ); // Just a separator
 
 	// Typical users shouldn't need to use this functionality - so debug only ATM
 	if ( vik_debug ) {
-		item = gtk_image_menu_item_new_with_mnemonic ( _("_Flush Memory Cache") );
-		gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU) );
-		g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_flush_memory), values );
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show ( item );
+		(void)vu_menu_add_item ( menu, _("_Flush Memory Cache"), GTK_STOCK_REMOVE, G_CALLBACK(mapnik_layer_flush_memory), values );
 	}
 
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_REFRESH, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_reload), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_REFRESH, G_CALLBACK(mapnik_layer_reload), values );
 
 	if ( g_strcmp0 ("", vml->filename_css) ) {
-		item = gtk_image_menu_item_new_with_mnemonic ( _("_Run Carto Command") );
-		gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU) );
-		g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_carto), values );
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show ( item );
+		(void)vu_menu_add_item ( menu, _("_Run Carto Command"), GTK_STOCK_EXECUTE, G_CALLBACK(mapnik_layer_carto), values );
 	}
 
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_INFO, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_information), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
-
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_ABOUT, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_about), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_INFO, G_CALLBACK(mapnik_layer_information), values );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_ABOUT, G_CALLBACK(mapnik_layer_about), values );
 }
 
 /**
@@ -1121,8 +1110,8 @@ static void mapnik_layer_tile_info ( VikMapnikLayer *vml )
 		g_array_append_val ( array, rendmsg );
 	}
 
-	a_dialog_list (  VIK_GTK_WINDOW_FROM_LAYER(vml), _("Tile Information"), array, 5 );
-	g_array_free ( array, FALSE );
+	a_dialog_list ( VIK_GTK_WINDOW_FROM_LAYER(vml), _("Tile Information"), array, 5 );
+	(void)g_array_free ( array, FALSE );
 
 	g_free ( rendmsg );
 	g_free ( timemsg );
@@ -1130,32 +1119,24 @@ static void mapnik_layer_tile_info ( VikMapnikLayer *vml )
 	g_free ( filename );
 }
 
-static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp )
+static VikLayerToolFuncStatus mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *event, VikViewport *vvp )
 {
 	if ( !vml )
-		return FALSE;
+		return VIK_LAYER_TOOL_IGNORED;
 	if ( event->button == 3 ) {
 		vik_viewport_screen_to_coord ( vvp, MAX(0, event->x), MAX(0, event->y), &vml->rerender_ul );
 		vml->rerender_zoom = vik_viewport_get_zoom ( vvp );
 
 		if ( ! vml->right_click_menu ) {
-			GtkWidget *item;
 			vml->right_click_menu = gtk_menu_new ();
-
-			item = gtk_image_menu_item_new_with_mnemonic ( _("_Rerender Tile") );
-			gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU) );
-			g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_rerender), vml );
-			gtk_menu_shell_append ( GTK_MENU_SHELL(vml->right_click_menu), item );
-
-			item = gtk_image_menu_item_new_with_mnemonic ( _("_Info") );
-			gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU) );
-			g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_tile_info), vml );
-			gtk_menu_shell_append ( GTK_MENU_SHELL(vml->right_click_menu), item );
+			GtkMenu *menu = GTK_MENU(vml->right_click_menu); // local rename
+			(void)vu_menu_add_item ( menu, _("_Rerender Tile"), GTK_STOCK_REFRESH, G_CALLBACK(mapnik_layer_rerender), vml );
+			(void)vu_menu_add_item ( menu, _("_Info"), GTK_STOCK_INFO, G_CALLBACK(mapnik_layer_tile_info), vml );
 		}
 
 		gtk_menu_popup ( GTK_MENU(vml->right_click_menu), NULL, NULL, NULL, NULL, event->button, event->time );
 		gtk_widget_show_all ( GTK_WIDGET(vml->right_click_menu) );
 	}
 
-	return FALSE;
+	return VIK_LAYER_TOOL_IGNORED;
 }
