@@ -22,7 +22,7 @@
 /**
  * SECTION:vikroutingwebengine
  * @short_description: A generic class for WEB based routing engine
- * 
+ *
  * The #VikRoutingWebEngine class handles WEB based
  * routing engine.
  */
@@ -52,7 +52,7 @@ typedef struct _VikRoutingWebEnginePrivate VikRoutingWebEnginePrivate;
 struct _VikRoutingWebEnginePrivate
 {
 	gchar *url_base;
-	
+
 	/* LatLon */
 	gchar *url_start_ll_fmt;
 	gchar *url_stop_ll_fmt;
@@ -66,7 +66,8 @@ struct _VikRoutingWebEnginePrivate
 	DownloadFileOptions options;
 };
 
-#define VIK_ROUTING_WEB_ENGINE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIK_ROUTING_WEB_ENGINE_TYPE, VikRoutingWebEnginePrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (VikRoutingWebEngine, vik_routing_web_engine, VIK_ROUTING_ENGINE_TYPE)
+#define VIK_ROUTING_WEB_ENGINE_PRIVATE(o)  (vik_routing_web_engine_get_instance_private (VIK_ROUTING_WEB_ENGINE(o)))
 
 /* properties */
 enum
@@ -74,7 +75,7 @@ enum
   PROP_0,
 
   PROP_URL_BASE,
-  
+
   /* LatLon */
   PROP_URL_START_LL,
   PROP_URL_STOP_LL,
@@ -86,11 +87,11 @@ enum
   PROP_URL_STOP_DIR,
 
   PROP_REFERER,
+  PROP_USER_AGENT,
   PROP_FOLLOW_LOCATION,
+  PROP_CUSTOM_HTTP_HEADERS,
 };
 
-G_DEFINE_TYPE (VikRoutingWebEngine, vik_routing_web_engine, VIK_ROUTING_ENGINE_TYPE)
-
 static void
 vik_routing_web_engine_set_property (GObject      *object,
                           guint         property_id,
@@ -140,10 +141,27 @@ vik_routing_web_engine_set_property (GObject      *object,
       priv->options.referer = g_value_dup_string (value);
       break;
 
+    case PROP_USER_AGENT:
+      g_free (priv->options.user_agent);
+      priv->options.user_agent = g_value_dup_string (value);
+      break;
+
     case PROP_FOLLOW_LOCATION:
       priv->options.follow_location = g_value_get_long (value);
       break;
 
+    case PROP_CUSTOM_HTTP_HEADERS:
+      {
+      g_free (priv->options.custom_http_headers);
+      const gchar *str = g_value_get_string ( value );
+      // Convert the literal characters '\'+'n' into actual newlines '\n'
+      if ( str )
+        priv->options.custom_http_headers = g_strcompress ( str );
+      else
+        priv->options.custom_http_headers = NULL;
+      }
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -193,10 +211,18 @@ vik_routing_web_engine_get_property (GObject    *object,
       g_value_set_string (value, priv->options.referer);
       break;
 
+    case PROP_USER_AGENT:
+      g_value_set_string (value, priv->options.user_agent);
+      break;
+
     case PROP_FOLLOW_LOCATION:
       g_value_set_long (value, priv->options.follow_location);
       break;
 
+    case PROP_CUSTOM_HTTP_HEADERS:
+      g_value_set_string (value, priv->options.custom_http_headers);
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -235,7 +261,7 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
                                "<no-set>" /* default value */,
                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_URL_BASE, pspec);
-  
+
 
   /**
    * VikRoutingWebEngine:url-start-ll:
@@ -261,7 +287,7 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
                                 TRUE/* default value */,
                                 G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_URL_LL_LAT_FIRST, pspec);
-   
+
 
   /**
    * VikRoutingWebEngine:url-stop-ll:
@@ -300,7 +326,7 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
                                NULL /* default value */,
                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_URL_START_DIR, pspec);
-    
+
 
   /**
    * VikRoutingWebEngine:url-stop-dir:
@@ -328,21 +354,45 @@ static void vik_routing_web_engine_class_init ( VikRoutingWebEngineClass *klass
   g_object_class_install_property (object_class, PROP_REFERER, pspec);
 
 
+  /**
+   * VikRoutingWebEngine:user-agent:
+   *
+   * The User Agent string to use in HTTP request.
+   */
+  pspec = g_param_spec_string ("user-agent",
+                               "User Agent",
+                               "The User Agent string to send in the request",
+                               NULL, // default value
+                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_USER_AGENT, pspec);
+
+
   /**
    * VikRoutingWebEngine:follow-location:
    *
-   * Specifies the number of retries to follow a redirect while downloading a page.
+   * Specifies the number of retries to follow a redirect while downloading a URL.
    */
   pspec = g_param_spec_long ("follow-location",
                              "Follow location",
-                             "Specifies the number of retries to follow a redirect while downloading a page",
-                             0  /* minimum value */,
+                             "Specifies the number of retries to follow a redirect while downloading a URL",
+                             -1,  // minimum value (unlimited)
                              G_MAXLONG /* maximum value */,
-                             2  /* default value */,
+                             -1,  // default value - automatically follow location with unlimited redirects
                              G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_FOLLOW_LOCATION, pspec);
 
-  g_type_class_add_private (klass, sizeof (VikRoutingWebEnginePrivate));
+
+  /**
+   * VikRoutingWebEngine:custom-http-headers:
+   *
+   * Define custom HTTP headers to use in requests.
+   */
+  pspec = g_param_spec_string ("custom-http-headers",
+                               "Custom HTTP Headers",
+                               "Custom HTTP Headers, use '\n' to separate multiple headers",
+                               NULL, // default value
+                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_CUSTOM_HTTP_HEADERS, pspec);
 }
 
 static void vik_routing_web_engine_init ( VikRoutingWebEngine *self )
@@ -350,7 +400,7 @@ static void vik_routing_web_engine_init ( VikRoutingWebEngine *self )
   VikRoutingWebEnginePrivate *priv = VIK_ROUTING_WEB_ENGINE_PRIVATE ( self );
 
   priv->url_base = NULL;
-  
+
   /* LatLon */
   priv->url_start_ll_fmt = NULL;
   priv->url_stop_ll_fmt = NULL;
@@ -361,11 +411,15 @@ static void vik_routing_web_engine_init ( VikRoutingWebEngine *self )
   priv->url_start_dir_fmt = NULL;
   priv->url_stop_dir_fmt = NULL;
 
+  /* Options */
   priv->options.referer = NULL;
-  priv->options.follow_location = 0;
+  priv->options.follow_location = -1;
+  priv->options.expiry_age = 0;
   priv->options.check_file = NULL;
   priv->options.check_file_server_time = FALSE;
   priv->options.use_etag = FALSE;
+  priv->options.user_agent = NULL;
+  priv->options.custom_http_headers = NULL;
 }
 
 static void vik_routing_web_engine_finalize ( GObject *gob )
@@ -374,7 +428,7 @@ static void vik_routing_web_engine_finalize ( GObject *gob )
 
   g_free (priv->url_base);
   priv->url_base = NULL;
-  
+
   /* LatLon */
   g_free (priv->url_start_ll_fmt);
   priv->url_start_ll_fmt = NULL;
@@ -390,9 +444,13 @@ static void vik_routing_web_engine_finalize ( GObject *gob )
   g_free (priv->url_stop_dir_fmt);
   priv->url_stop_dir_fmt = NULL;
 
+  /* Options */
   g_free (priv->options.referer);
   priv->options.referer = NULL;
-
+  g_free (priv->options.user_agent);
+  priv->options.user_agent = NULL;
+  g_free (priv->options.custom_http_headers);
+  priv->options.custom_http_headers = NULL;
   G_OBJECT_CLASS (vik_routing_web_engine_parent_class)->finalize(gob);
 }
 
@@ -400,30 +458,30 @@ static DownloadFileOptions *
 vik_routing_web_engine_get_download_options ( VikRoutingEngine *self )
 {
 	g_return_val_if_fail (VIK_IS_ROUTING_WEB_ENGINE(self), NULL);
-	
+
 	VikRoutingWebEnginePrivate *priv = VIK_ROUTING_WEB_ENGINE_PRIVATE(self);
-	
+
 	return &(priv->options);
 }
 
 static gchar *
 substitute_latlon ( const gchar *fmt, struct LatLon ll )
 {
-	gchar lat[G_ASCII_DTOSTR_BUF_SIZE], lon[G_ASCII_DTOSTR_BUF_SIZE];
-	gchar *substituted = g_strdup_printf(fmt,
-                          g_ascii_dtostr (lat, G_ASCII_DTOSTR_BUF_SIZE, (gdouble) ll.lat),
-                          g_ascii_dtostr (lon, G_ASCII_DTOSTR_BUF_SIZE, (gdouble) ll.lon));
-	return substituted;
+  gchar lat[COORDS_STR_BUFFER_SIZE], lon[COORDS_STR_BUFFER_SIZE];
+  a_coords_dtostr_buffer ( ll.lat, lat );
+  a_coords_dtostr_buffer ( ll.lon, lon );
+  gchar *substituted = g_strdup_printf( fmt, lat, lon );
+  return substituted;
 }
 
 static gchar *
 substitute_lonlat ( const gchar *fmt, struct LatLon ll )
 {
-	gchar lat[G_ASCII_DTOSTR_BUF_SIZE], lon[G_ASCII_DTOSTR_BUF_SIZE];
-	gchar *substituted = g_strdup_printf(fmt,
-                          g_ascii_dtostr (lon, G_ASCII_DTOSTR_BUF_SIZE, (gdouble) ll.lon),
-                          g_ascii_dtostr (lat, G_ASCII_DTOSTR_BUF_SIZE, (gdouble) ll.lat));
-	return substituted;
+  gchar lat[COORDS_STR_BUFFER_SIZE], lon[COORDS_STR_BUFFER_SIZE];
+  a_coords_dtostr_buffer ( ll.lat, lat );
+  a_coords_dtostr_buffer ( ll.lon, lon );
+  gchar *substituted = g_strdup_printf( fmt, lon, lat );
+  return substituted;
 }
 
 static gchar *
@@ -432,7 +490,7 @@ vik_routing_web_engine_get_url_for_coords ( VikRoutingEngine *self, struct LatLo
 	gchar *startURL;
 	gchar *endURL;
 	gchar *url;
-	
+
 	g_return_val_if_fail ( VIK_IS_ROUTING_WEB_ENGINE (self), NULL);
 
 	VikRoutingWebEnginePrivate *priv = VIK_ROUTING_WEB_ENGINE_PRIVATE ( self );
@@ -462,7 +520,7 @@ vik_routing_web_engine_find ( VikRoutingEngine *self, VikTrwLayer *vtl, struct L
   gchar *uri = vik_routing_web_engine_get_url_for_coords(self, start, end);
 
   DownloadFileOptions *options = vik_routing_web_engine_get_download_options(self);
-  
+
   gchar *format = vik_routing_engine_get_format ( self );
   ProcessOptions po = { NULL, NULL, format, uri, NULL, NULL };
   gboolean ret = a_babel_convert_from ( vtl, &po, NULL, NULL, options );
@@ -528,14 +586,14 @@ _append_stringified_coords ( gpointer data, gpointer user_data )
 {
   VikTrackpoint *vtp = (VikTrackpoint*)data;
   struct _append_ctx *ctx = (struct _append_ctx*)user_data;
-  
+
   /* Stringify coordinate */
   struct LatLon position;
   vik_coord_to_latlon ( &(vtp->coord), &position );
   gchar *string = (ctx->priv->url_ll_lat_first_fmt) ?
 	  	substitute_latlon ( ctx->priv->url_via_ll_fmt, position ):
 	  	substitute_lonlat ( ctx->priv->url_via_ll_fmt, position );
-  
+
   /* Append */
   ctx->urlParts[ctx->nb] = string;
   ctx->nb++;
