@@ -2,32 +2,32 @@
 /*
  * viking
  * Copyright (C) 2009, Guilhem Bonnefille <guilhem.bonnefille@gmail.com>
- * 
+ *
  * viking is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * viking is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
- 
+
  /**
   * SECTION:vikslippymapsource
   * @short_description: the class for SlippyMap oriented map sources
-  * 
+  *
   * The #VikSlippyMapSource class handles slippy map oriented map sources.
   * The related service is tile oriented, à la Google.
-  * 
+  *
   * The tiles are in 'google spherical mercator', which is
   * basically a mercator projection that assumes a spherical earth.
   * http://docs.openlayers.org/library/spherical_mercator.html
-  * 
+  *
   * Such service is also a type of TMS (Tile Map Service) as defined in
   * OSGeo's wiki.
   * http://wiki.osgeo.org/wiki/Tile_Map_Service_Specification
@@ -35,12 +35,12 @@
   * corner.
   * Following this specification, the protocol handled by this class
   * follows the global-mercator profile.
-  * 
+  *
   * You can also find many interesting information on the OSM's wiki.
   * http://wiki.openstreetmap.org/wiki/Slippy_map_tilenames
   * http://wiki.openstreetmap.org/wiki/Setting_up_TMS
   */
-  
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -64,13 +64,15 @@ static gdouble _get_lon_max(VikMapSource *self );
 
 static gchar *_get_uri( VikMapSourceDefault *self, MapCoord *src );
 static gchar *_get_hostname( VikMapSourceDefault *self );
-static DownloadFileOptions *_get_download_options( VikMapSourceDefault *self );
+static DownloadFileOptions *_get_download_options( VikMapSourceDefault *self, MapCoord *src );
+static void _set_expiry_age( VikMapSourceDefault *self, guint age );
 
 typedef struct _VikSlippyMapSourcePrivate VikSlippyMapSourcePrivate;
 struct _VikSlippyMapSourcePrivate
 {
   gchar *hostname;
   gchar *url;
+  gchar *custom_http_headers;
   DownloadFileOptions options;
   // NB Probably best to keep the above fields in same order to be common across Slippy, TMS & WMS map definitions
   guint zoom_min; // TMS Zoom level: 0 = Whole World // http://wiki.openstreetmap.org/wiki/Slippy_map_tilenames
@@ -86,7 +88,8 @@ struct _VikSlippyMapSourcePrivate
   gboolean switch_xy;
 };
 
-#define VIK_SLIPPY_MAP_SOURCE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIK_TYPE_SLIPPY_MAP_SOURCE, VikSlippyMapSourcePrivate))
+G_DEFINE_TYPE_WITH_PRIVATE (VikSlippyMapSource, vik_slippy_map_source, VIK_TYPE_MAP_SOURCE_DEFAULT);
+#define VIK_SLIPPY_MAP_SOURCE_PRIVATE(o)  (vik_slippy_map_source_get_instance_private (VIK_SLIPPY_MAP_SOURCE(o)))
 
 /* properties */
 enum
@@ -95,6 +98,7 @@ enum
 
   PROP_HOSTNAME,
   PROP_URL,
+  PROP_CUSTOM_HTTP_HEADERS,
   PROP_ZOOM_MIN,
   PROP_ZOOM_MAX,
   PROP_LAT_MIN,
@@ -102,6 +106,7 @@ enum
   PROP_LON_MIN,
   PROP_LON_MAX,
   PROP_REFERER,
+  PROP_USER_AGENT,
   PROP_FOLLOW_LOCATION,
   PROP_CHECK_FILE_SERVER_TIME,
   PROP_USE_ETAG,
@@ -111,8 +116,6 @@ enum
   PROP_SWITCH_XY,
 };
 
-G_DEFINE_TYPE (VikSlippyMapSource, vik_slippy_map_source, VIK_TYPE_MAP_SOURCE_DEFAULT);
-
 static void
 vik_slippy_map_source_init (VikSlippyMapSource *self)
 {
@@ -121,6 +124,7 @@ vik_slippy_map_source_init (VikSlippyMapSource *self)
 
   priv->hostname = NULL;
   priv->url = NULL;
+  priv->custom_http_headers = NULL;
   priv->zoom_min = 0;
   priv->zoom_max = 18;
   priv->lat_min = -90.0;
@@ -128,10 +132,13 @@ vik_slippy_map_source_init (VikSlippyMapSource *self)
   priv->lon_min = -180.0;
   priv->lon_max = 180.0;
   priv->options.referer = NULL;
-  priv->options.follow_location = 0;
+  priv->options.follow_location = -1;
+  priv->options.expiry_age = ONE_WEEK_SECS;
   priv->options.check_file = a_check_map_file;
-  priv->options.check_file_server_time = FALSE;
+  priv->options.check_file_server_time = TRUE;
   priv->options.use_etag = FALSE;
+  priv->options.user_agent = NULL;
+  priv->options.custom_http_headers = NULL;
   priv->is_direct_file_access = FALSE;
   priv->is_mbtiles = FALSE;
   priv->is_osm_meta_tiles = FALSE;
@@ -148,15 +155,20 @@ vik_slippy_map_source_init (VikSlippyMapSource *self)
 static void
 vik_slippy_map_source_finalize (GObject *object)
 {
-  VikSlippyMapSource *self = VIK_SLIPPY_MAP_SOURCE (object);
-  VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE (self);
+  VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE (object);
 
   g_free (priv->hostname);
   priv->hostname = NULL;
   g_free (priv->url);
   priv->url = NULL;
+  g_free (priv->custom_http_headers);
+  priv->custom_http_headers = NULL;
   g_free (priv->options.referer);
   priv->options.referer = NULL;
+  g_free (priv->options.user_agent);
+  priv->options.user_agent = NULL;
+  g_free (priv->options.custom_http_headers);
+  priv->options.custom_http_headers = NULL;
 
   G_OBJECT_CLASS (vik_slippy_map_source_parent_class)->finalize (object);
 }
@@ -167,8 +179,7 @@ vik_slippy_map_source_set_property (GObject      *object,
                                     const GValue *value,
                                     GParamSpec   *pspec)
 {
-  VikSlippyMapSource *self = VIK_SLIPPY_MAP_SOURCE (object);
-  VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE (self);
+  VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE (object);
 
   switch (property_id)
     {
@@ -182,6 +193,18 @@ vik_slippy_map_source_set_property (GObject      *object,
       priv->url = g_value_dup_string (value);
       break;
 
+    case PROP_CUSTOM_HTTP_HEADERS:
+      {
+      g_free (priv->custom_http_headers);
+      const gchar *str = g_value_get_string ( value );
+      // Convert the literal characters '\'+'n' into actual newlines '\n'
+      if ( str )
+        priv->custom_http_headers = g_strcompress ( str );
+      else
+        priv->custom_http_headers = NULL;
+      }
+      break;
+
     case PROP_ZOOM_MIN:
       priv->zoom_min = g_value_get_uint (value);
       break;
@@ -211,6 +234,11 @@ vik_slippy_map_source_set_property (GObject      *object,
       priv->options.referer = g_value_dup_string (value);
       break;
 
+    case PROP_USER_AGENT:
+      g_free (priv->options.user_agent);
+      priv->options.user_agent = g_value_dup_string (value);
+      break;
+
     case PROP_FOLLOW_LOCATION:
       priv->options.follow_location = g_value_get_long (value);
       break;
@@ -252,8 +280,7 @@ vik_slippy_map_source_get_property (GObject    *object,
                                     GValue     *value,
                                     GParamSpec *pspec)
 {
-  VikSlippyMapSource *self = VIK_SLIPPY_MAP_SOURCE (object);
-  VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE (self);
+  VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE (object);
 
   switch (property_id)
     {
@@ -265,6 +292,10 @@ vik_slippy_map_source_get_property (GObject    *object,
       g_value_set_string (value, priv->url);
       break;
 
+    case PROP_CUSTOM_HTTP_HEADERS:
+      g_value_set_string (value, priv->custom_http_headers);
+	  break;
+
     case PROP_ZOOM_MIN:
       g_value_set_uint (value, priv->zoom_min);
       break;
@@ -293,6 +324,10 @@ vik_slippy_map_source_get_property (GObject    *object,
       g_value_set_string (value, priv->options.referer);
       break;
 
+    case PROP_USER_AGENT:
+      g_value_set_string (value, priv->options.user_agent);
+      break;
+
     case PROP_FOLLOW_LOCATION:
       g_value_set_long (value, priv->options.follow_location);
       break;
@@ -300,7 +335,7 @@ vik_slippy_map_source_get_property (GObject    *object,
     case PROP_CHECK_FILE_SERVER_TIME:
       g_value_set_boolean (value, priv->options.check_file_server_time);
       break;
-	  
+
     case PROP_USE_ETAG:
       g_value_set_boolean (value, priv->options.use_etag);
       break;
@@ -335,7 +370,7 @@ vik_slippy_map_source_class_init (VikSlippyMapSourceClass *klass)
 	VikMapSourceClass* grandparent_class = VIK_MAP_SOURCE_CLASS (klass);
 	VikMapSourceDefaultClass* parent_class = VIK_MAP_SOURCE_DEFAULT_CLASS (klass);
 	GParamSpec *pspec = NULL;
-		
+
 	object_class->set_property = vik_slippy_map_source_set_property;
     object_class->get_property = vik_slippy_map_source_get_property;
 
@@ -356,6 +391,7 @@ vik_slippy_map_source_class_init (VikSlippyMapSourceClass *klass)
 	parent_class->get_uri = _get_uri;
 	parent_class->get_hostname = _get_hostname;
 	parent_class->get_download_options = _get_download_options;
+	parent_class->set_expiry_age = _set_expiry_age;
 
 	pspec = g_param_spec_string ("hostname",
 	                             "Hostname",
@@ -371,6 +407,13 @@ vik_slippy_map_source_class_init (VikSlippyMapSourceClass *klass)
 	                             G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_URL, pspec);
 
+	pspec = g_param_spec_string ("custom-http-headers",
+	                             "Custom HTTP Headers",
+	                             "Custom HTTP Headers, use '\n' to separate multiple headers",
+	                             NULL /* default value */,
+	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_CUSTOM_HTTP_HEADERS, pspec);
+
 	pspec = g_param_spec_uint ("zoom-min",
 	                           "Minimum zoom",
 	                           "Minimum Zoom level supported by the map provider",
@@ -431,20 +474,27 @@ vik_slippy_map_source_class_init (VikSlippyMapSourceClass *klass)
 	                             NULL /* default value */,
 	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_REFERER, pspec);
-	
+
+	pspec = g_param_spec_string ("user-agent",
+	                             "User Agent",
+	                             "The User Agent string to send in the request",
+	                             NULL, // default value
+	                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_USER_AGENT, pspec);
+
 	pspec = g_param_spec_long ("follow-location",
 	                           "Follow location",
                                "Specifies the number of retries to follow a redirect while downloading a page",
-                               0  /* minimum value */,
+	                           -1, // minimum value (unlimited)
                                G_MAXLONG /* maximum value */,
-                               0  /* default value */,
+                               -1, // default value - automatically follow location with unlimited redirects
                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_FOLLOW_LOCATION, pspec);
-	
+
 	pspec = g_param_spec_boolean ("check-file-server-time",
 	                              "Check file server time",
                                   "Age of current cache before redownloading tile",
-                                  FALSE  /* default value */,
+                                  TRUE  /* default value */,
                                   G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_CHECK_FILE_SERVER_TIME, pspec);
 
@@ -483,8 +533,6 @@ vik_slippy_map_source_class_init (VikSlippyMapSourceClass *klass)
 	                              G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_SWITCH_XY, pspec);
 
-	g_type_class_add_private (klass, sizeof (VikSlippyMapSourcePrivate));
-	
 	object_class->finalize = vik_slippy_map_source_finalize;
 }
 
@@ -523,9 +571,7 @@ static gboolean
 _supports_download_only_new (VikMapSource *self)
 {
   g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
-	
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
-	
   return priv->options.check_file_server_time || priv->options.use_etag;
 }
 
@@ -535,7 +581,7 @@ _supports_download_only_new (VikMapSource *self)
 static guint8
 _get_zoom_min (VikMapSource *self)
 {
-  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
+  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), 0);
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
   return priv->zoom_min;
 }
@@ -546,7 +592,7 @@ _get_zoom_min (VikMapSource *self)
 static guint8
 _get_zoom_max (VikMapSource *self)
 {
-  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
+  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), 18);
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
   return priv->zoom_max;
 }
@@ -557,7 +603,7 @@ _get_zoom_max (VikMapSource *self)
 static gdouble
 _get_lat_min (VikMapSource *self)
 {
-  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
+  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), -90.0);
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
   return priv->lat_min;
 }
@@ -568,7 +614,7 @@ _get_lat_min (VikMapSource *self)
 static gdouble
 _get_lat_max (VikMapSource *self)
 {
-  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
+  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), 90.0);
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
   return priv->lat_max;
 }
@@ -579,7 +625,7 @@ _get_lat_max (VikMapSource *self)
 static gdouble
 _get_lon_min (VikMapSource *self)
 {
-  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
+  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), -180.0);
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
   return priv->lon_min;
 }
@@ -590,7 +636,7 @@ _get_lon_min (VikMapSource *self)
 static gdouble
 _get_lon_max (VikMapSource *self)
 {
-  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), FALSE);
+  g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), 180.0);
   VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
   return priv->lon_max;
 }
@@ -611,9 +657,11 @@ static gchar *
 _get_uri( VikMapSourceDefault *self, MapCoord *src )
 {
 	g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), NULL);
-	
 	VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
 
+	if ( !priv->url )
+		return NULL;
+
 	gchar *uri = NULL;
 	if ( priv->switch_xy )
 		// 'ARC GIS' Tile Server layout ordering
@@ -623,24 +671,53 @@ _get_uri( VikMapSourceDefault *self, MapCoord *src )
 		uri = g_strdup_printf (priv->url, 17 - src->scale, src->x, src->y);
 
 	return uri;
-} 
+}
 
 static gchar *
 _get_hostname( VikMapSourceDefault *self )
 {
 	g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), NULL);
-	
     VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
 	return g_strdup( priv->hostname );
 }
 
+/**
+ * The return value is allocated and contents must be freed after use
+ * see: a_download_file_options_free()
+ */
 static DownloadFileOptions *
-_get_download_options( VikMapSourceDefault *self )
+_get_download_options( VikMapSourceDefault *self, MapCoord *src )
 {
 	g_return_val_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self), NULL);
-	
 	VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
-	return &(priv->options);
+
+	DownloadFileOptions *dfo = g_malloc0 ( sizeof(DownloadFileOptions) );
+	dfo->check_file_server_time = priv->options.check_file_server_time;
+	dfo->use_etag = priv->options.use_etag;
+	dfo->referer = g_strdup ( priv->options.referer );
+	dfo->user_agent = g_strdup ( priv->options.user_agent );
+	dfo->follow_location = priv->options.follow_location;
+	if ( src )
+		if ( priv->custom_http_headers )
+			dfo->custom_http_headers = g_strdup_printf ( priv->custom_http_headers, 17-src->scale, src->x, src->y );
+		else
+			dfo->custom_http_headers = NULL;
+	else
+		dfo->custom_http_headers = g_strdup ( priv->custom_http_headers );
+	dfo->check_file = priv->options.check_file;
+	dfo->user_pass = g_strdup ( priv->options.user_pass );
+	dfo->convert_file = priv->options.convert_file;
+	dfo->expiry_age = priv->options.expiry_age;
+
+	return dfo;
+}
+
+static void
+_set_expiry_age( VikMapSourceDefault *self, guint age )
+{
+	g_return_if_fail (VIK_IS_SLIPPY_MAP_SOURCE(self));
+	VikSlippyMapSourcePrivate *priv = VIK_SLIPPY_MAP_SOURCE_PRIVATE(self);
+	priv->options.expiry_age = age;
 }
 
 VikSlippyMapSource *
