@@ -2,7 +2,7 @@
 /*
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
- * Copyright (C) 2013, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2013-2020, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,14 +19,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <math.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-
 #include "viking.h"
 #include "viktrwlayer_tracklist.h"
 #include "viktrwlayer_propwin.h"
@@ -34,6 +26,12 @@
 // Long formatted date+basic time - listing this way ensures the string comparison sort works - so no local type format %x or %c here!
 #define TRACK_LIST_DATE_FORMAT "%Y-%m-%d %H:%M"
 
+// For simplicity these are global values
+//  (i.e. would get shared between multiple windows)
+static gint width = 0;
+static gint height = 400;
+static gboolean configured = FALSE;
+
 /**
  * track_close_cb:
  *
@@ -46,27 +44,25 @@ static void track_close_cb ( GtkWidget *dialog, gint resp, GList *data )
 	gtk_widget_destroy (dialog);
 }
 
-/**
- * format_1f_cell_data_func:
- *
- * General purpose column double formatting
- *
- */
-static void format_1f_cell_data_func ( GtkTreeViewColumn *col,
-                                       GtkCellRenderer   *renderer,
-                                       GtkTreeModel      *model,
-                                       GtkTreeIter       *iter,
-                                       gpointer           user_data )
+static gboolean configure_event ( GtkWidget *widget, GdkEventConfigure *event, gpointer notused )
 {
-	gdouble value;
-	gchar buf[20];
-	gint column = GPOINTER_TO_INT (user_data);
-	gtk_tree_model_get ( model, iter, column, &value, -1 );
-	g_snprintf ( buf, sizeof(buf), "%.1f", value );
-	g_object_set ( renderer, "text", buf, NULL );
+	if ( !configured ) {
+		configured = TRUE;
+
+		// Allow sizing back down to a minimum
+		GdkGeometry geom = { event->width, event->height, 0, 0, 0, 0, 0, 0, 0, 0, GDK_GRAVITY_STATIC };
+		gtk_window_set_geometry_hints ( GTK_WINDOW(widget), widget, &geom, GDK_HINT_BASE_SIZE );
+
+		// Restore previous size
+		gtk_window_resize ( GTK_WINDOW(widget), width, height );
+  } else {
+		width = event->width;
+		height = event->height;
+  }
+  return FALSE;
 }
 
-#define TRK_LIST_COLS 11
+#define TRK_LIST_COLS 12
 #define TRK_COL_NUM TRK_LIST_COLS-1
 #define TRW_COL_NUM TRK_COL_NUM-1
 
@@ -209,6 +205,7 @@ static void trw_layer_track_view ( menu_array_values values )
 
 typedef struct {
   gboolean has_layer_names;
+  gboolean is_only_routes;
   GString *str;
 } copy_data_t;
 
@@ -219,21 +216,34 @@ static void copy_selection (GtkTreeModel *model,
 {
 	copy_data_t *cd = (copy_data_t*) data;
 
+	vik_units_speed_t speed_units = a_vik_get_units_speed ();
+
 	gchar* layername; gtk_tree_model_get ( model, iter, 0, &layername, -1 );
 	gchar* name; gtk_tree_model_get ( model, iter, 1, &name, -1 );
 	gchar* date; gtk_tree_model_get ( model, iter, 2, &date, -1 );
 	gdouble d1; gtk_tree_model_get ( model, iter, 4, &d1, -1 );
 	guint d2; gtk_tree_model_get ( model, iter, 5, &d2, -1 );
 	gdouble d3; gtk_tree_model_get ( model, iter, 6, &d3, -1 );
+	gchar buf3[16];
+	vu_speed_text_value ( buf3, sizeof(buf3), speed_units, d3, "%.1f" );
 	gdouble d4; gtk_tree_model_get ( model, iter, 7, &d4, -1 );
+	gchar buf4[16];
+	vu_speed_text_value ( buf4, sizeof(buf4), speed_units, d4, "%.1f" );
 	gint d5; gtk_tree_model_get ( model, iter, 8, &d5, -1 );
 	gchar sep = '\t'; // Could make this configurable - but simply always make it a tab character for now
 	// NB Even if the columns have been reordered - this copies it out only in the original default order
 	// if col 0 is displayed then also copy the layername
-	if ( cd->has_layer_names )
-		g_string_append_printf ( cd->str, "%s%c%s%c%s%c%.1f%c%d%c%.1f%c%.1f%c%d\n", layername, sep, name, sep, date, sep, d1, sep, d2, sep, d3, sep, d4, sep, d5 );
-	else
-		g_string_append_printf ( cd->str, "%s%c%s%c%.1f%c%d%c%.1f%c%.1f%c%d\n", name, sep, date, sep, d1, sep, d2, sep, d3, sep, d4, sep, d5 );
+	if ( cd->has_layer_names ) {
+		if ( cd->is_only_routes )
+			g_string_append_printf ( cd->str, "%s%c%s%c%.1f%c%d\n", layername, sep, name, sep, d1, sep, d5 );
+		else
+			g_string_append_printf ( cd->str, "%s%c%s%c%s%c%.1f%c%d%c%s%c%s%c%d\n", layername, sep, name, sep, date, sep, d1, sep, d2, sep, buf3, sep, buf4, sep, d5 );
+	} else {
+		if ( cd->is_only_routes )
+			g_string_append_printf ( cd->str, "%s%c%.1f%c%d\n", name, sep, d1, sep, d5 );
+		else
+			g_string_append_printf ( cd->str, "%s%c%s%c%.1f%c%d%c%s%c%s%c%d\n", name, sep, date, sep, d1, sep, d2, sep, buf3, sep, buf4, sep, d5 );
+	}
 	g_free ( layername );
 	g_free ( name );
 	g_free ( date );
@@ -247,7 +257,9 @@ static void trw_layer_copy_selected ( GtkWidget *tree_view )
 	guint count = g_list_length ( gl );
 	g_list_free ( gl );
 	copy_data_t cd;
-	cd.has_layer_names = (count > TRK_LIST_COLS-3);
+	// Infer properties just from the number of columns shown
+	cd.has_layer_names = (count == 10 || count == 6);
+	cd.is_only_routes = (count == 6 || count == 5);
 	// Or use gtk_tree_view_column_get_visible()?
 	cd.str = g_string_new ( NULL );
 	gtk_tree_selection_selected_foreach ( selection, copy_selection, &cd );
@@ -259,18 +271,12 @@ static void trw_layer_copy_selected ( GtkWidget *tree_view )
 
 static void add_copy_menu_item ( GtkMenu *menu, GtkWidget *tree_view )
 {
-	GtkWidget *item = gtk_image_menu_item_new_with_mnemonic ( _("_Copy Data") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_MENU) );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_copy_selected), tree_view );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, _("_Copy Data"), GTK_STOCK_COPY, G_CALLBACK(trw_layer_copy_selected), tree_view );
 }
 
-static gboolean add_menu_items ( GtkMenu *menu, VikTrwLayer *vtl, VikTrack *trk, gpointer trk_uuid, VikViewport *vvp, GtkWidget *tree_view, gpointer data )
+static void add_menu_items ( GtkMenu *menu, VikTrwLayer *vtl, VikTrack *trk, gpointer trk_uuid, VikViewport *vvp, GtkWidget *tree_view, gpointer data )
 {
 	static menu_array_values values;
-	GtkWidget *item;
-
 	values[MA_VTL]       = vtl;
 	values[MA_TRK]       = trk;
 	values[MA_TRK_UUID]  = trk_uuid;
@@ -278,29 +284,12 @@ static gboolean add_menu_items ( GtkMenu *menu, VikTrwLayer *vtl, VikTrack *trk,
 	values[MA_TREEVIEW]  = tree_view;
 	values[MA_TRKS_LIST] = data;
 
-	/*
-	item = gtk_image_menu_item_new_with_mnemonic ( _("_Select") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_MENU) );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_track_select), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
-	*/
+	//(void)vu_menu_add_item ( menu, _("_Select"), GTK_STOCK_FIND, G_CALLBACK(trw_layer_track_select), values );
 
 	// ATM view auto selects, so don't bother with separate select menu entry
-	item = gtk_image_menu_item_new_with_mnemonic ( _("_View") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_MENU) );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_track_view), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
-
-	item = gtk_menu_item_new_with_mnemonic ( _("_Statistics") );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_track_stats), values );
-	gtk_menu_shell_append ( GTK_MENU_SHELL(menu), item );
-	gtk_widget_show ( item );
-
+	(void)vu_menu_add_item ( menu, _("_View"), GTK_STOCK_ZOOM_FIT, G_CALLBACK(trw_layer_track_view), values );
+	(void)vu_menu_add_item ( menu, _("_Statistics"), NULL, G_CALLBACK(trw_layer_track_stats), values );
 	add_copy_menu_item ( menu, tree_view );
-
-	return TRUE;
 }
 
 static gboolean trw_layer_track_menu_popup_multi  ( GtkWidget *tree_view,
@@ -310,7 +299,7 @@ static gboolean trw_layer_track_menu_popup_multi  ( GtkWidget *tree_view,
 	GtkWidget *menu = gtk_menu_new();
 
 	add_copy_menu_item ( GTK_MENU(menu), tree_view );
-
+	gtk_widget_show_all ( menu );
 	gtk_menu_popup ( GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time() );
 
 	return TRUE;
@@ -377,6 +366,7 @@ static gboolean trw_layer_track_menu_popup ( GtkWidget *tree_view,
 		                 tree_view,
 		                 data );
 
+		gtk_widget_show_all ( menu );
 		gtk_menu_popup ( GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time() );
 		return TRUE;
 	}
@@ -412,7 +402,7 @@ static gboolean trw_layer_track_button_pressed ( GtkWidget *tree_view,
  * Foreach entry we copy the various individual track properties into the tree store
  *  formatting & converting the internal values into something for display
  */
-static void trw_layer_track_list_add ( vik_trw_track_list_t *vtdl,
+static void trw_layer_track_list_add ( vik_trw_and_track_t *vtt,
                                        GtkTreeStore *store,
                                        vik_units_distance_t dist_units,
                                        vik_units_speed_t speed_units,
@@ -420,83 +410,49 @@ static void trw_layer_track_list_add ( vik_trw_track_list_t *vtdl,
                                        const gchar* date_format )
 {
 	GtkTreeIter t_iter;
-	VikTrack *trk = vtdl->trk;
-	VikTrwLayer *vtl = vtdl->vtl;
+	VikTrack *trk = vtt->trk;
+	VikTrwLayer *vtl = vtt->vtl;
 
-	gdouble trk_dist = vik_track_get_length ( trk );
 	// Store unit converted value
-	switch ( dist_units ) {
-	case VIK_UNITS_DISTANCE_MILES:
-		trk_dist = VIK_METERS_TO_MILES(trk_dist);
-		break;
-	case VIK_UNITS_DISTANCE_NAUTICAL_MILES:
-		trk_dist = VIK_METERS_TO_NAUTICAL_MILES(trk_dist);
-		break;
-	default:
-		trk_dist = trk_dist/1000.0;
-		break;
-	}
+	gdouble trk_dist = vu_distance_convert ( dist_units, vik_track_get_length(trk) );
 
 	// Get start date
 	gchar time_buf[32];
 	time_buf[0] = '\0';
-	if ( trk->trackpoints && VIK_TRACKPOINT(trk->trackpoints->data)->has_timestamp ) {
-
-#if GLIB_CHECK_VERSION(2,26,0)
-		GDateTime* gdt = g_date_time_new_from_unix_utc ( VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
-		gchar *time = g_date_time_format ( gdt, date_format );
+	if ( trk->trackpoints && !isnan(VIK_TRACKPOINT(trk->trackpoints->data)->timestamp) ) {
+		VikTrackpoint *tp = VIK_TRACKPOINT(trk->trackpoints->data);
+		time_t tt = tp->timestamp;
+		gchar *time = vu_get_time_string ( &tt, date_format, &tp->coord, NULL );
 		g_strlcpy ( time_buf, time, sizeof(time_buf) );
 		g_free ( time );
-		g_date_time_unref ( gdt);
-#else
-		GDate* gdate_start = g_date_new ();
-		g_date_set_time_t ( gdate_start, VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
-		g_date_strftime ( time_buf, sizeof(time_buf), date_format, gdate_start );
-		g_date_free ( gdate_start );
-#endif
 	}
 
-	// NB: doesn't include aggegrate visibility
-	gboolean visible = VIK_LAYER(vtl)->visible && trk->visible;
-	visible = visible && (trk->is_route ? vik_trw_layer_get_routes_visibility(vtl) : vik_trw_layer_get_tracks_visibility(vtl));
+	gboolean visible = trk->visible && (trk->is_route ? vik_trw_layer_get_routes_visibility(vtl) : vik_trw_layer_get_tracks_visibility(vtl));
+	visible = visible && vik_treeview_item_get_visible_tree ( VIK_LAYER(vtl)->vt, &(VIK_LAYER(vtl)->iter) );
 
 	guint trk_len_time = 0; // In minutes
 	if ( trk->trackpoints ) {
-		time_t t1, t2;
+		gdouble t1, t2;
 		t1 = VIK_TRACKPOINT(g_list_first(trk->trackpoints)->data)->timestamp;
 		t2 = VIK_TRACKPOINT(g_list_last(trk->trackpoints)->data)->timestamp;
-		trk_len_time = (int)round(labs(t2-t1)/60.0);
+		if ( !isnan(t1) && !isnan(t2) )
+			trk_len_time = (int)round(fabs(t2-t1)/60.0);
 	}
 
 	gdouble av_speed = 0.0;
 	gdouble max_speed = 0.0;
 	gdouble max_alt = 0.0;
 
-	av_speed = vik_track_get_average_speed ( trk );
-	switch (speed_units) {
-	case VIK_UNITS_SPEED_KILOMETRES_PER_HOUR: av_speed = VIK_MPS_TO_KPH(av_speed); break;
-	case VIK_UNITS_SPEED_MILES_PER_HOUR:      av_speed = VIK_MPS_TO_MPH(av_speed); break;
-	case VIK_UNITS_SPEED_KNOTS:               av_speed = VIK_MPS_TO_KNOTS(av_speed); break;
-	case VIK_UNITS_SPEED_SECONDS_PER_KM:      av_speed = VIK_MPS_TO_PACE_SPK(av_speed); break;
-	case VIK_UNITS_SPEED_MINUTES_PER_KM:      av_speed = VIK_MPS_TO_PACE_MPK(av_speed); break;
-	case VIK_UNITS_SPEED_SECONDS_PER_MILE:    av_speed = VIK_MPS_TO_PACE_SPM(av_speed); break;
-	case VIK_UNITS_SPEED_MINUTES_PER_MILE:    av_speed = VIK_MPS_TO_PACE_MPM(av_speed); break;
+	// Routes clearly don't have speeds
+	if ( !trk->is_route ) {
+		av_speed = vik_track_get_average_speed ( trk );
+		av_speed = vu_speed_convert ( speed_units, av_speed );
 
-	default: // VIK_UNITS_SPEED_METRES_PER_SECOND therefore no change
-		break;
-	}
-
-	max_speed = vik_track_get_max_speed ( trk );
-	switch (speed_units) {
-	case VIK_UNITS_SPEED_KILOMETRES_PER_HOUR: max_speed = VIK_MPS_TO_KPH(max_speed); break;
-	case VIK_UNITS_SPEED_MILES_PER_HOUR:      max_speed = VIK_MPS_TO_MPH(max_speed); break;
-	case VIK_UNITS_SPEED_KNOTS:               max_speed = VIK_MPS_TO_KNOTS(max_speed); break;
-	case VIK_UNITS_SPEED_SECONDS_PER_KM:      max_speed = VIK_MPS_TO_PACE_SPK(max_speed); break;
-	case VIK_UNITS_SPEED_MINUTES_PER_KM:      max_speed = VIK_MPS_TO_PACE_MPK(max_speed); break;
-	case VIK_UNITS_SPEED_SECONDS_PER_MILE:    max_speed = VIK_MPS_TO_PACE_SPM(max_speed); break;
-	case VIK_UNITS_SPEED_MINUTES_PER_MILE:    max_speed = VIK_MPS_TO_PACE_MPM(max_speed); break;
-	default: // VIK_UNITS_SPEED_METRES_PER_SECOND therefore no change
-		break;
+		max_speed = vu_track_get_max_speed ( trk, vik_trw_layer_get_prefer_gps_speed(vtl) );
+		if ( isnan(max_speed) )
+			max_speed = 0.0;
+		else
+			max_speed = vu_speed_convert ( speed_units, max_speed );
 	}
 
 	// TODO - make this a function to get min / max values?
@@ -506,7 +462,7 @@ static void trw_layer_track_list_add ( vik_trw_track_list_t *vtdl,
 		max_alt = -1000;
 		guint i;
 		for ( i=0; i < 500; i++ ) {
-			if ( altitudes[i] != VIK_DEFAULT_ALTITUDE ) {
+			if ( !isnan(altitudes[i]) ) {
 				if ( altitudes[i] > max_alt )
 					max_alt = altitudes[i];
 			}
@@ -532,19 +488,71 @@ static void trw_layer_track_list_add ( vik_trw_track_list_t *vtdl,
 	                     6, av_speed,
 	                     7, max_speed,
 	                     8, (gint)round(max_alt),
+	                     9, trk->is_route,
 	                     TRW_COL_NUM, vtl,
 	                     TRK_COL_NUM, trk,
 	                     -1 );
 }
 
-static GtkTreeViewColumn *my_new_column_text ( const gchar *title, GtkCellRenderer *renderer, GtkWidget *view, gint column_runner )
+static gboolean
+filter_tracks_cb (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
-	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes ( title, renderer, "text", column_runner, NULL );
-	gtk_tree_view_column_set_sort_column_id ( column, column_runner );
-	gtk_tree_view_append_column ( GTK_TREE_VIEW(view), column );
-	gtk_tree_view_column_set_reorderable ( column, TRUE );
-	gtk_tree_view_column_set_resizable ( column, TRUE );
-	return column;
+	const gchar *filter = gtk_entry_get_text ( GTK_ENTRY(data) );
+
+	if ( filter == NULL || strlen ( filter ) == 0 )
+		return TRUE;
+
+	gchar *filter_case = g_utf8_casefold ( filter, -1 );
+
+	VikTrack *trk;
+	gtk_tree_model_get ( model, iter, TRK_COL_NUM, &trk, -1 );
+
+	if ( trk == NULL )
+		return FALSE;
+
+	gboolean visible = FALSE;
+
+	gchar *layer_name;
+	gtk_tree_model_get ( model, iter, 0, &layer_name, -1 );
+	if ( layer_name != NULL ) {
+		gchar *layer_name_case = g_utf8_casefold ( layer_name, -1 );
+		visible |= ( strstr ( layer_name_case, filter_case ) != NULL );
+		g_free ( layer_name_case );
+	}
+	g_free ( layer_name );
+
+	gchar *trk_name_case = g_utf8_casefold ( trk->name, -1 );
+	visible |= ( strstr ( trk_name_case, filter_case ) != NULL );
+	g_free ( trk_name_case );
+
+	gchar *time_buf;
+	gtk_tree_model_get ( model, iter, 2, &time_buf, -1 );
+	visible |= (time_buf != NULL && strstr ( time_buf, filter_case )  != NULL);
+	g_free ( time_buf );
+
+	if ( trk->comment != NULL ) {
+		gchar *trk_comment_case = g_utf8_casefold ( trk->comment, -1 );
+		visible |= ( strstr ( trk_comment_case, filter_case ) != NULL );
+		g_free ( trk_comment_case );
+	}
+
+	if ( trk->description != NULL ) {
+		gchar *trk_description_case = g_utf8_casefold ( trk->description, -1 );
+		visible |= ( strstr ( trk_description_case, filter_case ) != NULL );
+		g_free ( trk_description_case );
+	}
+
+	g_free ( filter_case );
+
+	return visible;
+}
+
+static void
+filter_changed_cb (GtkEntry   *entry,
+                   GParamSpec *pspec,
+                   gpointer   data)
+{
+    gtk_tree_model_filter_refilter( GTK_TREE_MODEL_FILTER(data) );
 }
 
 /**
@@ -575,8 +583,9 @@ static void vik_trw_layer_track_list_internal ( GtkWidget *dialog,
 	                                           G_TYPE_DOUBLE,    // 6: Av. Speed
 	                                           G_TYPE_DOUBLE,    // 7: Max Speed
 	                                           G_TYPE_INT,       // 8: Max Height
-	                                           G_TYPE_POINTER,   // 9: TrackWaypoint Layer pointer
-	                                           G_TYPE_POINTER ); // 10: Track pointer
+	                                           G_TYPE_BOOLEAN,   // 9: Is Route
+	                                           G_TYPE_POINTER,   // 10: TrackWaypoint Layer pointer
+	                                           G_TYPE_POINTER ); // 11: Track pointer
 
 	//gtk_tree_selection_set_select_function ( gtk_tree_view_get_selection (GTK_TREE_VIEW(vt)), vik_treeview_selection_filter, vt, NULL );
 
@@ -590,9 +599,11 @@ static void vik_trw_layer_track_list_internal ( GtkWidget *dialog,
 	if ( !a_settings_get_string ( VIK_SETTINGS_LIST_DATE_FORMAT, &date_format ) )
 		date_format = g_strdup ( TRACK_LIST_DATE_FORMAT );
 
+	gboolean is_only_routes = TRUE;
 	GList *gl = tracks_and_layers;
 	while ( gl ) {
-		trw_layer_track_list_add ( (vik_trw_track_list_t*)gl->data, store, dist_units, speed_units, height_units, date_format );
+		trw_layer_track_list_add ( (vik_trw_and_track_t*)gl->data, store, dist_units, speed_units, height_units, date_format );
+		is_only_routes = is_only_routes & ((vik_trw_and_track_t*)gl->data)->trk->is_route;
 		gl = g_list_next ( gl );
 	}
 	g_free ( date_format );
@@ -610,22 +621,28 @@ static void vik_trw_layer_track_list_internal ( GtkWidget *dialog,
 	gint column_runner = 0;
 	if ( show_layer_names ) {
 		// Insert column for the layer name when viewing multi layers
-		column = my_new_column_text ( _("Layer"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Layer"), renderer, view, column_runner++ );
 		gtk_tree_view_column_set_expand ( column, TRUE );
+		gtk_tree_view_column_set_reorderable ( column, TRUE );
 		// remember the layer column so we can sort by it later
 		sort_by_column = column;
 	}
 	else
 		column_runner++;
 
-	column = my_new_column_text ( _("Name"), renderer, view, column_runner++ );
+	column = ui_new_column_text ( _("Name"), renderer, view, column_runner++ );
 	gtk_tree_view_column_set_expand ( column, TRUE );
+	gtk_tree_view_column_set_reorderable ( column, TRUE );
 	if ( !show_layer_names )
 		// remember the name column so we can sort by it later
 		sort_by_column = column;
 
-	column = my_new_column_text ( _("Date"), renderer, view, column_runner++ );
-	gtk_tree_view_column_set_expand ( column, TRUE );
+	if ( !is_only_routes ) {
+		column = ui_new_column_text ( _("Date"), renderer, view, column_runner++ );
+		gtk_tree_view_column_set_reorderable ( column, TRUE );
+		gtk_tree_view_column_set_expand ( column, TRUE );
+	} else
+		column_runner++;
 
 	GtkCellRenderer *renderer_toggle = gtk_cell_renderer_toggle_new ();
 	column = gtk_tree_view_column_new_with_attributes ( _("Visible"), renderer_toggle, "active", column_runner, NULL );
@@ -636,51 +653,58 @@ static void vik_trw_layer_track_list_internal ( GtkWidget *dialog,
 
 	switch ( dist_units ) {
 	case VIK_UNITS_DISTANCE_MILES:
-		column = my_new_column_text ( _("Distance\n(miles)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Distance\n(miles)"), renderer, view, column_runner++ );
 		break;
 	case VIK_UNITS_DISTANCE_NAUTICAL_MILES:
-		column = my_new_column_text ( _("Distance\n(NM)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Distance\n(NM)"), renderer, view, column_runner++ );
 		break;
 	default:
-		column = my_new_column_text ( _("Distance\n(km)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Distance\n(km)"), renderer, view, column_runner++ );
 		break;
 	}
 	// Apply own formatting of the data
-	gtk_tree_view_column_set_cell_data_func ( column, renderer, format_1f_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL);
+	gtk_tree_view_column_set_cell_data_func ( column, renderer, ui_format_1f_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL);
+	gtk_tree_view_column_set_reorderable ( column, TRUE );
 
-	(void)my_new_column_text ( _("Length\n(minutes)"), renderer, view, column_runner++ );
+	if ( !is_only_routes ) {
+		column = ui_new_column_text ( _("Length\n(minutes)"), renderer, view, column_runner++ );
+		gtk_tree_view_column_set_reorderable ( column, TRUE );
 
-	gchar *spd_units = NULL;
-	switch (speed_units) {
-	case VIK_UNITS_SPEED_KILOMETRES_PER_HOUR: spd_units = g_strdup (_("km/h")); break;
-	case VIK_UNITS_SPEED_MILES_PER_HOUR:      spd_units = g_strdup (_("mph")); break;
-	case VIK_UNITS_SPEED_KNOTS:               spd_units = g_strdup (_("knots")); break;
-	case VIK_UNITS_SPEED_SECONDS_PER_KM:      spd_units = g_strdup (_("s/km")); break;
-	case VIK_UNITS_SPEED_MINUTES_PER_KM:      spd_units = g_strdup (_("min/km")); break;
-	case VIK_UNITS_SPEED_SECONDS_PER_MILE:    spd_units = g_strdup (_("sec/mi")); break;
-	case VIK_UNITS_SPEED_MINUTES_PER_MILE:    spd_units = g_strdup (_("min/mi")); break;
-	// VIK_UNITS_SPEED_METRES_PER_SECOND:
-	default:                                  spd_units = g_strdup (_("m/s")); break;
-	}
+		gchar *spd_units = vu_speed_units_text ( speed_units );
 
-	gchar *title = g_strdup_printf ( _("Av. Speed\n(%s)"), spd_units );
-	column = my_new_column_text ( title, renderer, view, column_runner++ );
-	g_free ( title );
-	gtk_tree_view_column_set_cell_data_func ( column, renderer, format_1f_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL); // Apply own formatting of the data
+		gchar *title = g_strdup_printf ( _("Av. Speed\n(%s)"), spd_units );
+		column = ui_new_column_text ( title, renderer, view, column_runner++ );
+		g_free ( title );
+		gtk_tree_view_column_set_cell_data_func ( column, renderer, vu_format_speed_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL); // Apply own formatting of the data
+		gtk_tree_view_column_set_reorderable ( column, TRUE );
 
-	title = g_strdup_printf ( _("Max Speed\n(%s)"), spd_units );
-	column = my_new_column_text ( title, renderer, view, column_runner++ );
-	gtk_tree_view_column_set_cell_data_func ( column, renderer, format_1f_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL); // Apply own formatting of the data
+		title = g_strdup_printf ( _("Max Speed\n(%s)"), spd_units );
+		column = ui_new_column_text ( title, renderer, view, column_runner++ );
+		gtk_tree_view_column_set_cell_data_func ( column, renderer, vu_format_speed_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL); // Apply own formatting of the data
+		gtk_tree_view_column_set_reorderable ( column, TRUE );
 
-	g_free ( title );
-	g_free ( spd_units );
+		g_free ( title );
+		g_free ( spd_units );
+	} else
+		column_runner += 3;
 
 	if ( height_units == VIK_UNITS_HEIGHT_FEET )
-		(void)my_new_column_text ( _("Max Height\n(Feet)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Max Height\n(Feet)"), renderer, view, column_runner++ );
 	else
-		(void)my_new_column_text ( _("Max Height\n(Metres)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Max Height\n(Metres)"), renderer, view, column_runner++ );
+	gtk_tree_view_column_set_reorderable ( column, TRUE );
 
-	gtk_tree_view_set_model ( GTK_TREE_VIEW(view), GTK_TREE_MODEL(store) );
+	GtkCellRenderer *renderer_toggle_rt = gtk_cell_renderer_toggle_new ();
+	column = gtk_tree_view_column_new_with_attributes ( _("Route"), renderer_toggle_rt, "active", column_runner, NULL );
+	gtk_tree_view_column_set_reorderable ( column, TRUE );
+	gtk_tree_view_column_set_sort_column_id ( column, column_runner );
+	gtk_tree_view_append_column ( GTK_TREE_VIEW(view), column );
+	column_runner++;
+
+	GtkTreeModelFilter *model = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new ( GTK_TREE_MODEL(store), NULL));
+	GtkTreeModelSort *sorted = GTK_TREE_MODEL_SORT(gtk_tree_model_sort_new_with_model ( GTK_TREE_MODEL(model) ));
+
+	gtk_tree_view_set_model ( GTK_TREE_VIEW(view), GTK_TREE_MODEL(sorted) );
 	gtk_tree_selection_set_mode ( gtk_tree_view_get_selection(GTK_TREE_VIEW(view)), GTK_SELECTION_MULTIPLE );
 	gtk_tree_view_set_rules_hint ( GTK_TREE_VIEW(view), TRUE );
 
@@ -698,15 +722,24 @@ static void vik_trw_layer_track_list_internal ( GtkWidget *dialog,
 	g_signal_connect ( view, "popup-menu", G_CALLBACK(trw_layer_track_menu_popup), tracks_and_layers );
 	g_signal_connect ( view, "button-press-event", G_CALLBACK(trw_layer_track_button_pressed), tracks_and_layers );
 
-	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), scrolledwindow, TRUE, TRUE, 0);
+	GtkWidget *filter_entry = ui_entry_new( NULL, GTK_ENTRY_ICON_SECONDARY );
+	g_signal_connect ( filter_entry, "notify::text", G_CALLBACK(filter_changed_cb), model );
+	gtk_tree_model_filter_set_visible_func ( model, filter_tracks_cb, filter_entry, NULL );
+
+	GtkWidget *filter_label = gtk_label_new ( _("Filter") );
+	g_object_set ( filter_label, "has-tooltip", TRUE, NULL );
+	g_signal_connect ( filter_label, "query-tooltip", G_CALLBACK(ui_tree_model_number_tooltip_cb), model );
+
+	GtkWidget *filter_box = gtk_hbox_new( FALSE, 10 );
+	gtk_box_pack_start (GTK_BOX(filter_box), filter_label, FALSE, TRUE, 10);
+	gtk_box_pack_start (GTK_BOX(filter_box), filter_entry, TRUE, TRUE, 10);
+
+	GtkBox *vbox = GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
+	gtk_box_pack_start (vbox, scrolledwindow, TRUE, TRUE, 0);
+	gtk_box_pack_end (vbox, filter_box, FALSE, TRUE, 10);
 
 	// Set ordering of the initial view by one of the name columns
 	gtk_tree_view_column_clicked ( sort_by_column );
-
-	// Ensure a reasonable number of items are shown
-	//  TODO: may be save window size, column order, sorted by between invocations.
-	// Gtk too stupid to work out best size so need to tell it.
-	gtk_window_set_default_size ( GTK_WINDOW(dialog), show_layer_names ? 900 : 700, 400 );
 }
 
 
@@ -741,9 +774,14 @@ void vik_trw_layer_track_list_show_dialog ( gchar *title,
 	// Use response to close the dialog with tidy up
 	g_signal_connect ( G_OBJECT(dialog), "response", G_CALLBACK(track_close_cb), gl );
 
+	configured = FALSE;
+	g_signal_connect ( G_OBJECT(dialog), "configure-event", G_CALLBACK(configure_event), NULL );
+
 	gtk_widget_show_all ( dialog );
-	// Yes - set the size *AGAIN* - this time widgets are expanded nicely
-	gtk_window_resize ( GTK_WINDOW(dialog), show_layer_names ? 1000 : 800, 400 );
+
+	// Ensure a reasonable number of items are shown
+	if ( width == 0 )
+		width = show_layer_names ? 1000 : 850;
 
 	// ATM lock out on dialog run - to prevent list contents being manipulated in other parts of the GUI whilst shown here.
 	gtk_dialog_run (GTK_DIALOG (dialog));
