@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
 # Inspired by MBUtils:
 #  http://github.com/mapbox/mbutil
@@ -41,16 +41,30 @@ def mbtiles_connect(mbtiles_file):
         logger.exception(e)
         sys.exit(1)
 
+# v1.1 style
+def mbtiles_metadata(cur, name):
+    cur.execute("""insert into metadata (name, value) values (?, ?);""",
+                ("name", name))
+    cur.execute("""insert into metadata (name, value) values (?, ?);""",
+                ("type", "baselayer"))
+    cur.execute("""insert into metadata (name, value) values (?, ?);""",
+                ("version", 1))
+    cur.execute("""insert into metadata (name, value) values (?, ?);""",
+                ("description", "Created by Viking - http://viking.sf.net/"))
+    cur.execute("""insert into metadata (name, value) values (?, ?);""",
+                ("format", "png"))
+
 def optimize_connection(cur):
     cur.execute("""PRAGMA synchronous=0""")
     cur.execute("""PRAGMA locking_mode=EXCLUSIVE""")
     cur.execute("""PRAGMA journal_mode=DELETE""")
 
-def write_database(cur):
-    logger.debug('analyzing db')
-    cur.execute("""ANALYZE;""")
+def write_database(con):
+    con.commit()
 
 def optimize_database(cur):
+    logger.debug('analyzing db')
+    cur.execute("""ANALYZE;""")
     logger.debug('cleaning db')
     cur.execute("""VACUUM;""")
 
@@ -70,11 +84,12 @@ def osm_to_mbtiles(directory_path, mbtiles_file, **kwargs):
     cur = con.cursor()
     optimize_connection(cur)
     mbtiles_setup(cur)
+    mbtiles_metadata(cur, mbtiles_file)
     image_format = 'png'
     count = 0
     start_time = time.time()
     msg = ""
-    onlydigits_re = re.compile ('^\d+$')
+    onlydigits_re = re.compile (r'^\d+$')
 
     for zoomDir in getDirs(directory_path):
         digitsz = onlydigits_re.match(zoomDir);
@@ -109,10 +124,11 @@ def osm_to_mbtiles(directory_path, mbtiles_file, **kwargs):
     if count == 0:
         print ("No tiles inserted")
     else:
-        write_database(cur)
+        write_database(con)
         if not kwargs.get('nooptimize'):
             sys.stdout.write("Optimizing...\n")
             optimize_database(con)
+    con.close()
     return
 
 # Based on disk_to_mbtiles in mbutil
@@ -126,12 +142,12 @@ def vikcache_to_mbtiles(directory_path, mbtiles_file, **kwargs):
     count = 0
     start_time = time.time()
     msg = ""
-    onlydigits_re = re.compile ('^\d+$')
+    onlydigits_re = re.compile (r'^\d+$')
 
     #print ('tileid ' + kwargs.get('tileid'))
     # Need to split tDddsDdzD
     #  note zoom level can be negative hence the '-?' term
-    p = re.compile ('^t'+kwargs.get('tileid')+'s(-?\d+)z\d+$')
+    p = re.compile ('^t'+kwargs.get('tileid')+r's(-?\d+)z\d+$')
     for ff in os.listdir(directory_path):
         # Find only dirs related to this tileset
         m = p.match(ff);
@@ -174,10 +190,11 @@ def vikcache_to_mbtiles(directory_path, mbtiles_file, **kwargs):
     if count == 0:
         print ("No tiles inserted. NB This method only works with the Legacy Viking cache layout")
     else:
-        write_database(cur)
+        write_database(con)
         if not kwargs.get('nooptimize'):
             sys.stdout.write("Optimizing...\n")
             optimize_database(con)
+    con.close()
     return
 
 def mbtiles_to_vikcache(mbtiles_file, directory_path, **kwargs):
@@ -226,9 +243,9 @@ def mbtiles_to_vikcache(mbtiles_file, directory_path, **kwargs):
 def cache_converter_to_osm (vc_path, target_path, **kwargs):
     msg = ''
     count = 0
-    onlydigits_re = re.compile ('^\d+$')
-    etag_re = re.compile ('\.etag$')
-    path_re = re.compile ('^t'+kwargs.get('tileid')+'s(-?\d+)z\d+$')
+    onlydigits_re = re.compile (r'^\d+$')
+    etag_re = re.compile (r'\.etag$')
+    path_re = re.compile ('^t'+kwargs.get('tileid')+r's(-?\d+)z\d+$')
 
     if not os.path.isdir(target_path):
         os.makedirs(target_path)
@@ -294,7 +311,7 @@ def cache_converter_to_osm (vc_path, target_path, **kwargs):
 def cache_converter_to_viking (osm_path, target_path, **kwargs):
     msg = ''
     count = 0
-    ispng = re.compile ('\.png$')
+    ispng = re.compile (r'\.png$')
 
     if not os.path.isdir(target_path):
         os.makedirs(target_path)
@@ -358,6 +375,8 @@ def get_tile_path (tid):
         return "Mapbox-Outdoors"
     elif tile_id == 212:
         return "Bing-Aerial"
+    elif tile_id == 901:
+        return "OpenTopoMap"
     # Default extension Map ids (from data/maps.xml)
     elif tile_id == 29:
         return "CalTopo"
@@ -373,7 +392,7 @@ def get_tile_path (tid):
 ##
 parser = OptionParser(usage="""usage: %prog -m <mode> [options] input output
 
-When either the input or output refers to a Viking legacy cache ('vcl'), is it the root directory of the cache, typically ~/.viking-maps
+When either the input or output refers to a Viking legacy cache ('vlc'), is it the root directory of the cache, typically ~/.viking-maps
 
 Examples:
 
@@ -400,9 +419,9 @@ Here one must specify the output directory name explicitly and set your maps.xml
 
 parser.add_option('-t', '--tileid', dest='tileid',
     action="store",
-    help='''Tile id of Viking map cache to use (19 if not specified as this is Viking's default (MaqQuest))''',
+    help='''Tile id of Viking map cache to use (13 if not specified as this is Viking's default (OSM Mapnik))''',
     type='string',
-    default='19')
+    default='13')
 
 parser.add_option('-n', '--nooptimize', dest='nooptimize',
     action="store_true",
@@ -456,7 +475,7 @@ elif options.__dict__.get('mode') == 'mbtiles2vlc':
         mbtiles_to_vikcache(in_fd, out_fd, **options.__dict__)
 elif options.__dict__.get('mode') == 'vlc2osm':
     # Main forward conversion
-    is_default_re = re.compile ("\.viking-maps\/?$")
+    is_default_re = re.compile (r"\.viking-maps\/?$")
     out_fd2 = is_default_re.search(out_fd)
     if out_fd2:
         # Auto append default tile name to the path
